# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tink Registry."""

from typing import Any, Tuple, Type, TypeVar

from tink.proto import tink_pb2
from tink import _monitoring
from tink.core import _key_manager
from tink.core import _primitive_set
from tink.core import _primitive_wrapper
from tink.core import _tink_error

P = TypeVar('P')


class Registry:
  """A global container of key managers.

  Registry maps supported key types to a corresponding KeyManager object,
  which 'understands' the key type (i.e., the KeyManager can instantiate the
  primitive corresponding to given key, or can generate new keys of the
  supported key type). Keeping KeyManagers for all primitives in a single
  Registry (rather than having a separate KeyManager per primitive) enables
  modular construction of compound primitives from 'simple' ones,
  e.g., AES-CTR-HMAC AEAD encryption uses IND-CPA encryption and a MAC.

  Registry is initialized at startup, and is later used to instantiate
  primitives for given keys or keysets.
  """

  _key_managers = {}  # type: dict[str, Tuple[_key_manager.KeyManager, bool]]
  _wrappers = {}  # type: dict[Type, _primitive_wrapper.PrimitiveWrapper]

  @classmethod
  def reset(cls) -> None:
    """Resets the registry."""
    cls._key_managers = {}
    cls._wrappers = {}

  @classmethod
  def _key_manager_internal(
      cls, type_url: str
  ) -> Tuple[_key_manager.KeyManager, bool]:
    """Returns a key manager, new_key_allowed pair for the given type_url."""
    if type_url not in cls._key_managers:
      raise _tink_error.TinkError(
          'No manager for type {} has been registered.'.format(type_url)
      )
    return cls._key_managers[type_url]

  @classmethod
  def key_manager(cls, type_url: str) -> _key_manager.KeyManager:
    """Returns a key manager for the given type_url and primitive_class.

    Args:
      type_url: Key type string

    Returns:
      A KeyManager object
    """
    key_mgr, _ = cls._key_manager_internal(type_url)
    return key_mgr

  @classmethod
  def register_key_manager(
      cls, key_manager: _key_manager.KeyManager, new_key_allowed: bool = True
  ) -> None:
    """Tries to register a key_manager for the given key_manager.key_type().

    Args:
      key_manager: A KeyManager object
      new_key_allowed: If new_key_allowed is true, users can generate new keys
        with this manager using Registry.new_key()
    """
    key_managers = cls._key_managers
    type_url = key_manager.key_type()
    primitive_class = key_manager.primitive_class()

    if not key_manager.does_support(type_url):
      raise _tink_error.TinkError(
          'The manager does not support its own type {}.'.format(type_url)
      )

    if type_url in key_managers:
      existing, existing_new_key = key_managers[type_url]
      if (
          type(existing) != type(key_manager)  # pylint: disable=unidiomatic-typecheck
          or existing.primitive_class() != primitive_class
      ):
        raise _tink_error.TinkError(
            'A manager for type {} has been already registered.'.format(
                type_url
            )
        )
      else:
        if not existing_new_key and new_key_allowed:
          raise _tink_error.TinkError(
              (
                  'A manager for type {} has been already registered '
                  'with forbidden new key operation.'
              ).format(type_url)
          )
        key_managers[type_url] = (existing, new_key_allowed)
    else:
      key_managers[type_url] = (key_manager, new_key_allowed)

  @classmethod
  def primitive(cls, key_data: tink_pb2.KeyData, primitive_class: Type[P]) -> P:
    """Creates a new primitive for the key given in key_data.

    It looks up a KeyManager identified by key_data.type_url,
    and calls manager's primitive(key_data) method.

    Args:
      key_data: KeyData object
      primitive_class: The expected primitive class

    Returns:
      A primitive for the given key_data
    Raises:
      Error if primitive_class does not match the registered primitive class.
    """
    key_mgr = cls.key_manager(key_data.type_url)
    if key_mgr.primitive_class() != primitive_class:
      raise _tink_error.TinkError(
          'Wrong primitive class: type {} uses primitive {}, and not {}.'
          .format(
              key_data.type_url,
              key_mgr.primitive_class().__name__,
              primitive_class.__name__,
          )
      )
    return key_mgr.primitive(key_data)

  @classmethod
  def new_key_data(cls, key_template: tink_pb2.KeyTemplate) -> tink_pb2.KeyData:
    """Generates a new key for the specified key_template."""
    key_mgr, new_key_allowed = cls._key_manager_internal(key_template.type_url)

    if not new_key_allowed:
      raise _tink_error.TinkError(
          'KeyManager for type {} does not allow for creation of new keys.'
          .format(key_template.type_url)
      )

    return key_mgr.new_key_data(key_template)

  @classmethod
  def public_key_data(
      cls, private_key_data: tink_pb2.KeyData
  ) -> tink_pb2.KeyData:
    """Generates a new key for the specified key_template."""
    if (
        private_key_data.key_material_type
        != tink_pb2.KeyData.ASYMMETRIC_PRIVATE
    ):
      raise _tink_error.TinkError('The keyset contains a non-private key')
    key_mgr = cls.key_manager(private_key_data.type_url)
    if not isinstance(key_mgr, _key_manager.PrivateKeyManager):
      raise _tink_error.TinkError(
          'manager for key type {} is not a PrivateKeyManager'.format(
              private_key_data.type_url
          )
      )
    return key_mgr.public_key_data(private_key_data)

  @classmethod
  def register_primitive_wrapper(
      cls, wrapper: _primitive_wrapper.PrimitiveWrapper
  ) -> None:
    """Tries to register a PrimitiveWrapper.

    Args:
      wrapper: A PrimitiveWrapper object.

    Raises:
      TinkError if a different wrapper has already been registered for the same
      Primitive.
    """
    # pylint: disable-next=unidiomatic-typecheck
    if wrapper.primitive_class() in cls._wrappers and type(
        cls._wrappers[wrapper.primitive_class()]
    ) != type(wrapper):
      raise _tink_error.TinkError(
          'A wrapper for primitive {} has already been added.'.format(
              wrapper.primitive_class().__name__
          )
      )
    wrapped = wrapper.wrap(
        _primitive_set.PrimitiveSet(wrapper.input_primitive_class())
    )
    if not isinstance(wrapped, wrapper.primitive_class()):
      raise _tink_error.TinkError(
          'Wrapper for primitive {} generates incompatible primitive of type {}'
          .format(wrapper.primitive_class().__name__, type(wrapped).__name__)
      )
    cls._wrappers[wrapper.primitive_class()] = wrapper

  @classmethod
  def input_primitive_class(cls, primitive_class: Any) -> Any:
    """Returns the primitive class that gets wrapped into primitive_class.

    Args:
      primitive_class: Class of the output primitive of a wrapper.

    Returns:
      the primitive class that gets wrapped. This needs to be the type used
      in the primitive set.
    Raises:
      TinkError if no wrapper for this primitive class is registered.
    """
    if primitive_class not in cls._wrappers:
      raise _tink_error.TinkError(
          'No PrimitiveWrapper registered for primitive {}.'.format(
              primitive_class.__name__
          )
      )
    wrapper = cls._wrappers[primitive_class]
    return wrapper.input_primitive_class()

  @classmethod
  def _get_and_validate_wrapper(
      cls,
      primitive_set: _primitive_set.PrimitiveSet,
      primitive_class: Any,
  ) -> _primitive_wrapper.PrimitiveWrapper:
    """Returns and validates the wrapper for the given primitive class."""
    if primitive_class not in cls._wrappers:
      raise _tink_error.TinkError(
          'No PrimitiveWrapper registered for primitive {}.'.format(
              primitive_class.__name__
          )
      )
    wrapper = cls._wrappers[primitive_class]
    if primitive_set.primitive_class() != wrapper.input_primitive_class():
      raise _tink_error.TinkError(
          'Wrapper for primitive {} wraps type {}, but the primitive_set'
          'has type {}'.format(
              wrapper.primitive_class().__name__,
              wrapper.input_primitive_class().__name__,
              primitive_set.primitive_class().__name__,
          )
      )
    return wrapper

  @classmethod
  def wrap(
      cls, primitive_set: _primitive_set.PrimitiveSet, primitive_class: Type[P]
  ) -> P:
    """Wraps a set of primitives into a single primitive.

    Args:
      primitive_set: A PrimitiveSet object.
      primitive_class: Class of the output primitive.

    Returns:
      A primitive of type primitive_class that wraps the primitives in
      primitive_set.
    Raises:
      TinkError if no wrapper for this primitive class is registered or the type
      of the primitives in primitive_set are don't match the
      input_primitive_class of the wrapper.
    """
    wrapper = cls._get_and_validate_wrapper(primitive_set, primitive_class)
    return wrapper.wrap(primitive_set)

  @classmethod
  def _wrap_with_monitoring_info(
      cls,
      primitive_set: _primitive_set.PrimitiveSet,
      primitive_class: Type[P],
      monitoring_keyset_info: _monitoring.MonitoringKeySetInfo,
  ) -> P:
    """Same as `wrap`, but passes along monitoring information."""

    wrapper = cls._get_and_validate_wrapper(primitive_set, primitive_class)
    # pylint: disable-next=protected-access
    return wrapper._wrap_with_monitoring_info(
        primitive_set, monitoring_keyset_info
    )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/rsa_ssa_pkcs1.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/rsa_ssa_pkcs1.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tink.proto import common_pb2 as tink_dot_proto_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1etink/proto/rsa_ssa_pkcs1.proto\x12\x12google.crypto.tink\x1a\x17tink/proto/common.proto\"D\n\x11RsaSsaPkcs1Params\x12/\n\thash_type\x18\x01 \x01(\x0e\x32\x1c.google.crypto.tink.HashType\"t\n\x14RsaSsaPkcs1PublicKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x35\n\x06params\x18\x02 \x01(\x0b\x32%.google.crypto.tink.RsaSsaPkcs1Params\x12\t\n\x01n\x18\x03 \x01(\x0c\x12\t\n\x01\x65\x18\x04 \x01(\x0c\"\xac\x01\n\x15RsaSsaPkcs1PrivateKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12<\n\npublic_key\x18\x02 \x01(\x0b\x32(.google.crypto.tink.RsaSsaPkcs1PublicKey\x12\t\n\x01\x64\x18\x03 \x01(\x0c\x12\t\n\x01p\x18\x04 \x01(\x0c\x12\t\n\x01q\x18\x05 \x01(\x0c\x12\n\n\x02\x64p\x18\x06 \x01(\x0c\x12\n\n\x02\x64q\x18\x07 \x01(\x0c\x12\x0b\n\x03\x63rt\x18\x08 \x01(\x0c\"\x84\x01\n\x14RsaSsaPkcs1KeyFormat\x12\x35\n\x06params\x18\x01 \x01(\x0b\x32%.google.crypto.tink.RsaSsaPkcs1Params\x12\x1c\n\x14modulus_size_in_bits\x18\x02 \x01(\r\x12\x17\n\x0fpublic_exponent\x18\x03 \x01(\x0c\x42`\n\x1c\x63om.google.crypto.tink.protoP\x01Z>github.com/tink-crypto/tink-go/v2/proto/rsa_ssa_pkcs1_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.rsa_ssa_pkcs1_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z>github.com/tink-crypto/tink-go/v2/proto/rsa_ssa_pkcs1_go_proto'
  _globals['_RSASSAPKCS1PARAMS']._serialized_start=79
  _globals['_RSASSAPKCS1PARAMS']._serialized_end=147
  _globals['_RSASSAPKCS1PUBLICKEY']._serialized_start=149
  _globals['_RSASSAPKCS1PUBLICKEY']._serialized_end=265
  _globals['_RSASSAPKCS1PRIVATEKEY']._serialized_start=268
  _globals['_RSASSAPKCS1PRIVATEKEY']._serialized_end=440
  _globals['_RSASSAPKCS1KEYFORMAT']._serialized_start=443
  _globals['_RSASSAPKCS1KEYFORMAT']._serialized_end=575
# @@protoc_insertion_point(module_scope)



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tink package."""

from tink import _json_proto_keyset_format as json_proto_keyset_format
from tink import _keyset_handle
from tink import _keyset_reader
from tink import _keyset_writer
from tink import _kms_clients
from tink import _proto_keyset_format as proto_keyset_format
from tink import core

new_keyset_handle = _keyset_handle.new_keyset_handle
read_keyset_handle = _keyset_handle.read_keyset_handle
read_keyset_handle_with_associated_data = _keyset_handle.read_keyset_handle_with_associated_data
read_no_secret_keyset_handle = _keyset_handle.read_no_secret_keyset_handle
KeysetHandle = _keyset_handle.KeysetHandle

KeysetReader = _keyset_reader.KeysetReader
JsonKeysetReader = _keyset_reader.JsonKeysetReader
BinaryKeysetReader = _keyset_reader.BinaryKeysetReader

KeysetWriter = _keyset_writer.KeysetWriter
JsonKeysetWriter = _keyset_writer.JsonKeysetWriter
BinaryKeysetWriter = _keyset_writer.BinaryKeysetWriter

TinkError = core.TinkError

KeyAccess = core.KeyAccess
PUBLIC_KEY_ACCESS_TOKEN = _keyset_handle.PUBLIC_KEY_ACCESS_TOKEN
has_secret_key_access = _keyset_handle.has_secret_key_access

KmsClient = _kms_clients.KmsClient

# Deprecated. It is preferable to not register KMS clients. Instead, get the
# KMS AEAD with kms_aead = client.get_aead(key_uri) and then use it to encrypt
# a keyset with KeysetHandle.write, or to create an envelope AEAD using
# aead.KmsEnvelopeAead.
register_kms_client = _kms_clients.register_kms_client

# Deprecated. It is preferable to not register KMS clients.
kms_client_from_uri = _kms_clients.kms_client_from_uri



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/kms_aead.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/kms_aead.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19tink/proto/kms_aead.proto\x12\x12google.crypto.tink\"#\n\x10KmsAeadKeyFormat\x12\x0f\n\x07key_uri\x18\x01 \x01(\t\"S\n\nKmsAeadKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x34\n\x06params\x18\x02 \x01(\x0b\x32$.google.crypto.tink.KmsAeadKeyFormatB[\n\x1c\x63om.google.crypto.tink.protoP\x01Z9github.com/tink-crypto/tink-go/v2/proto/kms_aead_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.kms_aead_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z9github.com/tink-crypto/tink-go/v2/proto/kms_aead_go_proto'
  _globals['_KMSAEADKEYFORMAT']._serialized_start=49
  _globals['_KMSAEADKEYFORMAT']._serialized_end=84
  _globals['_KMSAEADKEY']._serialized_start=86
  _globals['_KMSAEADKEY']._serialized_end=169
# @@protoc_insertion_point(module_scope)



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Jwt package."""

import datetime
from typing import Dict, List, Mapping, Optional, Union, cast

from tink.jwt import _jwk_set_converter
from tink.jwt import _jwt_error
from tink.jwt import _jwt_hmac_key_manager
from tink.jwt import _jwt_key_templates
from tink.jwt import _jwt_mac
from tink.jwt import _jwt_mac_wrapper
from tink.jwt import _jwt_public_key_sign
from tink.jwt import _jwt_public_key_verify
from tink.jwt import _jwt_signature_key_manager
from tink.jwt import _jwt_signature_wrappers
from tink.jwt import _jwt_validator
from tink.jwt import _raw_jwt
from tink.jwt import _verified_jwt

JwtInvalidError = _jwt_error.JwtInvalidError
RawJwt = _raw_jwt.RawJwt
VerifiedJwt = _verified_jwt.VerifiedJwt
JwtValidator = _jwt_validator.JwtValidator
Claim = _raw_jwt.Claim
JwtMac = _jwt_mac.JwtMac
JwtPublicKeySign = _jwt_public_key_sign.JwtPublicKeySign
JwtPublicKeyVerify = _jwt_public_key_verify.JwtPublicKeyVerify

new_raw_jwt = _raw_jwt.new_raw_jwt
new_validator = _jwt_validator.new_validator

jwk_set_from_public_keyset_handle = _jwk_set_converter.from_public_keyset_handle
jwk_set_to_public_keyset_handle = _jwk_set_converter.to_public_keyset_handle

jwt_hs256_template = _jwt_key_templates.jwt_hs256_template
raw_jwt_hs256_template = _jwt_key_templates.raw_jwt_hs256_template
jwt_hs384_template = _jwt_key_templates.jwt_hs384_template
raw_jwt_hs384_template = _jwt_key_templates.raw_jwt_hs384_template
jwt_hs512_template = _jwt_key_templates.jwt_hs512_template
raw_jwt_hs512_template = _jwt_key_templates.raw_jwt_hs512_template
jwt_es256_template = _jwt_key_templates.jwt_es256_template
raw_jwt_es256_template = _jwt_key_templates.raw_jwt_es256_template
jwt_es384_template = _jwt_key_templates.jwt_es384_template
raw_jwt_es384_template = _jwt_key_templates.raw_jwt_es384_template
jwt_es512_template = _jwt_key_templates.jwt_es512_template
raw_jwt_es512_template = _jwt_key_templates.raw_jwt_es512_template
jwt_rs256_2048_f4_template = _jwt_key_templates.jwt_rs256_2048_f4_template
raw_jwt_rs256_2048_f4_template = _jwt_key_templates.raw_jwt_rs256_2048_f4_template
jwt_rs256_3072_f4_template = _jwt_key_templates.jwt_rs256_3072_f4_template
raw_jwt_rs256_3072_f4_template = _jwt_key_templates.raw_jwt_rs256_3072_f4_template
jwt_rs384_3072_f4_template = _jwt_key_templates.jwt_rs384_3072_f4_template
raw_jwt_rs384_3072_f4_template = _jwt_key_templates.raw_jwt_rs384_3072_f4_template
jwt_rs512_4096_f4_template = _jwt_key_templates.jwt_rs512_4096_f4_template
raw_jwt_rs512_4096_f4_template = _jwt_key_templates.raw_jwt_rs512_4096_f4_template
jwt_ps256_2048_f4_template = _jwt_key_templates.jwt_ps256_2048_f4_template
raw_jwt_ps256_2048_f4_template = _jwt_key_templates.raw_jwt_ps256_2048_f4_template
jwt_ps256_3072_f4_template = _jwt_key_templates.jwt_ps256_3072_f4_template
raw_jwt_ps256_3072_f4_template = _jwt_key_templates.raw_jwt_ps256_3072_f4_template
jwt_ps384_3072_f4_template = _jwt_key_templates.jwt_ps384_3072_f4_template
raw_jwt_ps384_3072_f4_template = _jwt_key_templates.raw_jwt_ps384_3072_f4_template
jwt_ps512_4096_f4_template = _jwt_key_templates.jwt_ps512_4096_f4_template
raw_jwt_ps512_4096_f4_template = _jwt_key_templates.raw_jwt_ps512_4096_f4_template


def register_jwt_mac() -> None:
  _jwt_hmac_key_manager.register()
  _jwt_mac_wrapper.register()


def register_jwt_signature() -> None:
  _jwt_signature_key_manager.register()
  _jwt_signature_wrappers.register()


# Deprecated. Use jwk_set_from_public_keyset_handle instead.
jwk_set_from_keyset_handle = _jwk_set_converter.from_keyset_handle
# Deprecated. Use jwk_set_to_public_keyset_handle instead.
jwk_set_to_keyset_handle = _jwk_set_converter.to_keyset_handle



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.jwt._jwt_key_templates."""

from absl.testing import absltest
from absl.testing import parameterized

import tink
from tink import jwt


def setUpModule():
  jwt.register_jwt_mac()
  jwt.register_jwt_signature()


class JwtKeyTemplatesTest(parameterized.TestCase):

  @parameterized.named_parameters([
      ('JWT_HS256', jwt.jwt_hs256_template()),
      ('JWT_HS256_RAW', jwt.raw_jwt_hs256_template()),
      ('JWT_HS384', jwt.jwt_hs384_template()),
      ('JWT_HS384_RAW', jwt.raw_jwt_hs384_template()),
      ('JWT_HS512', jwt.jwt_hs512_template()),
      ('JWT_HS512_RAW', jwt.raw_jwt_hs512_template()),
  ])
  def test_mac_success(self, key_template):
    keyset_handle = tink.new_keyset_handle(key_template)
    jwt_hmac = keyset_handle.primitive(jwt.JwtMac)
    token = jwt.new_raw_jwt(
        issuer='issuer', subject='subject', without_expiration=True)
    compact = jwt_hmac.compute_mac_and_encode(token)
    output_token = jwt_hmac.verify_mac_and_decode(
        compact,
        jwt.new_validator(
            expected_issuer='issuer',
            allow_missing_expiration=True))
    self.assertEqual(output_token.issuer(), token.issuer())
    self.assertEqual(output_token.subject(), token.subject())

  @parameterized.named_parameters([
      ('JWT_ES256', jwt.jwt_es256_template()),
      ('JWT_ES256_RAW', jwt.raw_jwt_es256_template()),
      ('JWT_ES384', jwt.jwt_es384_template()),
      ('JWT_ES384_RAW', jwt.raw_jwt_es384_template()),
      ('JWT_ES512', jwt.jwt_es512_template()),
      ('JWT_ES512_RAW', jwt.raw_jwt_es512_template()),
      ('JWT_RS256_2048_F4', jwt.jwt_rs256_2048_f4_template()),
      ('JWT_RS256_2048_F4_RAW', jwt.raw_jwt_rs256_2048_f4_template()),
      ('JWT_RS256_3072_F4', jwt.jwt_rs256_3072_f4_template()),
      ('JWT_RS256_3072_F4_RAW', jwt.raw_jwt_rs256_3072_f4_template()),
      ('JWT_RS384_3072_F4', jwt.jwt_rs384_3072_f4_template()),
      ('JWT_RS384_3072_F4_RAW', jwt.raw_jwt_rs384_3072_f4_template()),
      ('JWT_RS512_4096_F4', jwt.jwt_rs512_4096_f4_template()),
      ('JWT_RS512_4096_F4_RAW', jwt.raw_jwt_rs512_4096_f4_template()),
      ('JWT_PS256_2048_F4', jwt.jwt_ps256_2048_f4_template()),
      ('JWT_PS256_2048_F4_RAW', jwt.raw_jwt_ps256_2048_f4_template()),
      ('JWT_PS256_3072_F4', jwt.jwt_ps256_3072_f4_template()),
      ('JWT_PS256_3072_F4_RAW', jwt.raw_jwt_ps256_3072_f4_template()),
      ('JWT_PS384_3072_F4', jwt.jwt_ps384_3072_f4_template()),
      ('JWT_PS384_3072_F4_RAW', jwt.raw_jwt_ps384_3072_f4_template()),
      ('JWT_PS512_4096_F4', jwt.jwt_ps512_4096_f4_template()),
      ('JWT_PS512_4096_F4_RAW', jwt.raw_jwt_ps512_4096_f4_template()),
  ])
  def test_new_keydata_primitive_success(self, template):
    private_handle = tink.new_keyset_handle(template)
    sign = private_handle.primitive(jwt.JwtPublicKeySign)
    verify = private_handle.public_keyset_handle().primitive(
        jwt.JwtPublicKeyVerify)
    raw_jwt = jwt.new_raw_jwt(
        issuer='issuer', subject='subject', without_expiration=True)
    compact = sign.sign_and_encode(raw_jwt)
    verified_jwt = verify.verify_and_decode(
        compact,
        jwt.new_validator(
            expected_issuer='issuer',
            allow_missing_expiration=True))
    self.assertEqual(verified_jwt.issuer(), 'issuer')
    self.assertEqual(verified_jwt.subject(), 'subject')

if __name__ == '__main__':
  absltest.main()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.integration.aws_kms_aead."""

import os

from absl.testing import absltest
import boto3

import tink
from tink import _kms_clients
from tink import aead
from tink.integration import awskms
from tink.integration.awskms import _aws_kms_client
from tink.testing import helper


CREDENTIAL_PATH = os.path.join(helper.tink_py_testdata_path(),
                               'aws/credentials.ini')

BAD_CREDENTIALS_PATH = os.path.join(helper.tink_py_testdata_path(),
                                    'aws/credentials_bad.ini')

KEY_URI = ('aws-kms://arn:aws:kms:us-east-2:235739564943:key/'
           '3ee50705-5a82-4f5b-9753-05c4f473922f')

# An alias for KEY_URI.
KEY_ALIAS_URI = ('aws-kms://arn:aws:kms:us-east-2:235739564943:alias/'
                 'unit-and-integration-testing')

KEY_URI_2 = ('aws-kms://arn:aws:kms:us-east-2:235739564943:key/'
             'b3ca2efd-a8fb-47f2-b541-7e20f8c5cd11')

GCP_KEY_URI = ('gcp-kms://projects/tink-test-infrastructure/locations/global/'
               'keyRings/unit-and-integration-testing/cryptoKeys/aead-key')


def setUpModule():
  aead.register()


class AwsKmsAeadTest(absltest.TestCase):

  def tearDown(self):
    super().tearDown()
    _kms_clients.reset_kms_clients()

  def test_encrypt_decrypt(self):
    aws_client = awskms.AwsKmsClient(KEY_URI, CREDENTIAL_PATH)
    aws_aead = aws_client.get_aead(KEY_URI)

    plaintext = b'hello'
    associated_data = b'world'
    ciphertext = aws_aead.encrypt(plaintext, associated_data)
    self.assertEqual(plaintext, aws_aead.decrypt(ciphertext, associated_data))

    plaintext = b'hello'
    ciphertext = aws_aead.encrypt(plaintext, b'')
    self.assertEqual(plaintext, aws_aead.decrypt(ciphertext, b''))

  def test_encrypt_decrypt_with_key_alias(self):
    aws_client = awskms.AwsKmsClient(KEY_ALIAS_URI, CREDENTIAL_PATH)
    aws_aead = aws_client.get_aead(KEY_ALIAS_URI)

    plaintext = b'hello'
    associated_data = b'world'
    ciphertext = aws_aead.encrypt(plaintext, associated_data)
    self.assertEqual(plaintext, aws_aead.decrypt(ciphertext, associated_data))

    plaintext = b'hello'
    ciphertext = aws_aead.encrypt(plaintext, b'')
    self.assertEqual(plaintext, aws_aead.decrypt(ciphertext, b''))

  def test_corrupted_ciphertext(self):
    aws_client = awskms.AwsKmsClient(KEY_URI, CREDENTIAL_PATH)
    aws_aead = aws_client.get_aead(KEY_URI)

    plaintext = b'helloworld'
    ciphertext = aws_aead.encrypt(plaintext, b'')
    self.assertEqual(plaintext, aws_aead.decrypt(ciphertext, b''))

    # Corrupt each byte once and check that decryption fails
    # NOTE: Skipping two bytes as they are malleable
    for byte_idx in [b for b in range(len(ciphertext)) if b not in [77, 123]]:
      tmp_ciphertext = list(ciphertext)
      tmp_ciphertext[byte_idx] ^= 1
      corrupted_ciphertext = bytes(tmp_ciphertext)
      with self.assertRaises(tink.TinkError):
        aws_aead.decrypt(corrupted_ciphertext, b'')

  def test_encrypt_with_bad_uri(self):
    with self.assertRaises(tink.TinkError):
      aws_client = awskms.AwsKmsClient(KEY_URI, CREDENTIAL_PATH)
      aws_client.get_aead(GCP_KEY_URI)

  def test_encrypt_with_bad_credentials(self):
    aws_client = awskms.AwsKmsClient(KEY_URI, BAD_CREDENTIALS_PATH)
    aws_aead = aws_client.get_aead(KEY_URI)

    plaintext = b'hello'
    associated_data = b'world'
    with self.assertRaises(tink.TinkError):
      aws_aead.encrypt(plaintext, associated_data)

  def test_new_client_get_aead(self):
    aws_access_key_id, aws_secret_access_key = _aws_kms_client._parse_config(
        CREDENTIAL_PATH
    )
    boto3_client = boto3.client(
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        region_name='us-east-2',
        service_name='kms',
    )

    aws_client = awskms.new_client(boto3_client=boto3_client)

    aws_aead = aws_client.get_aead(KEY_URI)
    ciphertext = aws_aead.encrypt(b'plaintext', b'associated_data')
    self.assertEqual(
        b'plaintext', aws_aead.decrypt(ciphertext, b'associated_data')
    )

    aws_aead_with_alias = aws_client.get_aead(KEY_ALIAS_URI)
    ciphertext = aws_aead_with_alias.encrypt(b'plaintext', b'associated_data')
    self.assertEqual(
        b'plaintext',
        aws_aead_with_alias.decrypt(ciphertext, b'associated_data'),
    )

    with self.assertRaises(tink.TinkError):
      aws_client.get_aead(GCP_KEY_URI)

  def test_client_registration(self):
    # Register AWS KMS Client bound to KEY_URI.
    awskms.AwsKmsClient.register_client(KEY_URI, CREDENTIAL_PATH)

    # Create a keyset handle for KEY_URI and use it.
    handle = tink.new_keyset_handle(
        aead.aead_key_templates.create_kms_aead_key_template(KEY_URI)
    )
    aws_aead = handle.primitive(aead.Aead)
    ciphertext = aws_aead.encrypt(b'plaintext', b'associated_data')
    self.assertEqual(
        b'plaintext', aws_aead.decrypt(ciphertext, b'associated_data')
    )

    # It fails for any other key URI.
    with self.assertRaises(tink.TinkError):
      handle2 = tink.new_keyset_handle(
          aead.aead_key_templates.create_kms_aead_key_template(KEY_URI_2)
      )
      gcp_aead = handle2.primitive(aead.Aead)
      gcp_aead.encrypt(b'plaintext', b'associated_data')

  def test_get_aead_is_compatible_with_kms_aead_key(self):
    # Get Aead directly from AwsKmsClient
    aws_aead = awskms.AwsKmsClient(KEY_URI, CREDENTIAL_PATH).get_aead(KEY_URI)

    # Use KmsAeadKey: Register client, create keyset and then Aead.
    awskms.AwsKmsClient.register_client(KEY_URI, CREDENTIAL_PATH)
    handle = tink.new_keyset_handle(
        aead.aead_key_templates.create_kms_aead_key_template(KEY_URI)
    )
    aead_from_kms_aead_key = handle.primitive(aead.Aead)

    # check that they are compatible.
    ciphertext = aws_aead.encrypt(b'plaintext', b'associated_data')
    self.assertEqual(
        b'plaintext',
        aead_from_kms_aead_key.decrypt(ciphertext, b'associated_data'),
    )

  def test_encrypt_with_default_credentials(self):
    # If no credentials_path is provided, this path here is used by default.
    os.environ['AWS_SHARED_CREDENTIALS_FILE'] = CREDENTIAL_PATH

    aws_client = awskms.AwsKmsClient(key_uri=KEY_URI, credentials_path=None)
    aws_aead = aws_client.get_aead(KEY_URI)

    ciphertext = aws_aead.encrypt(b'plaintext', b'associated_data')
    self.assertEqual(
        b'plaintext', aws_aead.decrypt(ciphertext, b'associated_data')
    )

    # creates a boto3 client using default credentials
    boto3_client = boto3.client(region_name='us-east-2', service_name='kms')
    aws_client2 = awskms.new_client(boto3_client=boto3_client, key_uri=KEY_URI)
    aws_aead2 = aws_client2.get_aead(KEY_URI)
    ciphertext2 = aws_aead2.encrypt(b'plaintext', b'associated_data')
    self.assertEqual(
        b'plaintext', aws_aead2.decrypt(ciphertext2, b'associated_data')
    )

    # check that aws_aead and aws_aead2 are compatible
    self.assertEqual(
        b'plaintext', aws_aead2.decrypt(ciphertext, b'associated_data')
    )
    self.assertEqual(
        b'plaintext', aws_aead.decrypt(ciphertext2, b'associated_data')
    )

  def test_server_side_key_commitment(self):
    key1_client = awskms.AwsKmsClient(KEY_URI, CREDENTIAL_PATH)
    key1_aead = key1_client.get_aead(KEY_URI)

    alias_client = awskms.AwsKmsClient(KEY_ALIAS_URI, CREDENTIAL_PATH)
    alias_aead = alias_client.get_aead(KEY_ALIAS_URI)

    key2_client = awskms.AwsKmsClient(KEY_URI_2, CREDENTIAL_PATH)
    key2_aead = key2_client.get_aead(KEY_URI_2)

    plaintext = b'hello'
    associated_data = b'world'

    # Confirm that KEY_URI and KEY_ALIAS_URI are interchangeable while the
    # KEY_ALIAS_URI continues to reference KEY_URI. This no longer holds if
    # KEY_ALIAS_URI is updated to reference a different key.
    for a in (key1_aead, alias_aead):
      ciphertext = a.encrypt(plaintext, associated_data)

      # Decrypt with KEY_URI.
      self.assertEqual(
          plaintext, key1_aead.decrypt(ciphertext, associated_data)
      )

      # Decrypt with KEY_ALIAS_URI.
      self.assertEqual(
          plaintext, alias_aead.decrypt(ciphertext, associated_data)
      )

      # Attempt to decrypt with KEY_URI_2.
      with self.assertRaises(tink.TinkError):
        key2_aead.decrypt(ciphertext, associated_data)

  def test_kms_behavior(self):
    aws_aead = awskms.AwsKmsClient(KEY_URI, CREDENTIAL_PATH).get_aead(KEY_URI)
    plaintext = b'hello'
    associated_data = b'world'
    ciphertext = aws_aead.encrypt(plaintext, associated_data)

    # NOTE: The following operations directly utilize the KMS client to verify
    # KMS behavior for requests not produced by this implementation (e.g. no
    # KeyId specified, inspecting response fields).

    encryption_context = _aws_kms_client._encryption_context(associated_data)

    # Verify key ID contained in the reponse. AWS KMS includes resolved key IDs
    # in responses, not aliases. This behavior precludes client-side key
    # commitment checking.
    response = aws_aead.client.decrypt(
        KeyId=_aws_kms_client._key_uri_to_key_arn(KEY_ALIAS_URI),
        CiphertextBlob=ciphertext,
        EncryptionContext=encryption_context,
    )
    self.assertEqual(plaintext, response['Plaintext'])
    self.assertEqual(
        _aws_kms_client._key_uri_to_key_arn(KEY_URI), response['KeyId']
    )

    # Decrypt without specifying a key ID in the request. Decrypt requests
    # produced by this library always include a key ID to trigger server side
    # key commitment checks.
    response = aws_aead.client.decrypt(
        CiphertextBlob=ciphertext,
        EncryptionContext=encryption_context,
    )
    self.assertEqual(plaintext, response['Plaintext'])


if __name__ == '__main__':
  absltest.main()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Reads Keysets from file."""

import abc

from google.protobuf import json_format
from google.protobuf import message
from tink.proto import tink_pb2
from tink import core


class KeysetReader(metaclass=abc.ABCMeta):
  """Reads a Keyset."""

  @abc.abstractmethod
  def read(self) -> tink_pb2.Keyset:
    """Reads and returns a (cleartext) tink_pb2.Keyset from its source."""
    raise NotImplementedError()

  @abc.abstractmethod
  def read_encrypted(self) -> tink_pb2.EncryptedKeyset:
    """Reads and returns an tink_pb2.EncryptedKeyset from its source."""
    raise NotImplementedError()


# Deprecated. Instead, use the parse functions in
# tink.json_proto_keyset_format.
class JsonKeysetReader(KeysetReader):
  """Reads a JSON Keyset."""

  def __init__(self, serialized_keyset: str):
    self._serialized_keyset = serialized_keyset

  def read(self) -> tink_pb2.Keyset:
    try:
      return json_format.Parse(self._serialized_keyset, tink_pb2.Keyset())
    except json_format.ParseError as e:
      raise core.TinkError(e)

  def read_encrypted(self) -> tink_pb2.EncryptedKeyset:
    try:
      return json_format.Parse(self._serialized_keyset,
                               tink_pb2.EncryptedKeyset())
    except json_format.ParseError as e:
      raise core.TinkError(e)


# Deprecated. Instead, use the parse functions in tink.proto_keyset_format.
class BinaryKeysetReader(KeysetReader):
  """Reads a binary Keyset."""

  def __init__(self, serialized_keyset: bytes):
    self._serialized_keyset = serialized_keyset

  def read(self) -> tink_pb2.Keyset:
    if not self._serialized_keyset:
      raise core.TinkError('No keyset found')
    try:
      return tink_pb2.Keyset.FromString(self._serialized_keyset)
    except message.DecodeError as e:
      raise core.TinkError(e)

  def read_encrypted(self) -> tink_pb2.EncryptedKeyset:
    if not self._serialized_keyset:
      raise core.TinkError('No keyset found')
    try:
      return tink_pb2.EncryptedKeyset.FromString(self._serialized_keyset)
    except message.DecodeError as e:
      raise core.TinkError(e)



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.hybrid_key_manager."""

from typing import cast
from absl.testing import absltest
from absl.testing import parameterized
from tink.proto import common_pb2
from tink.proto import ecies_aead_hkdf_pb2
from tink.proto import hpke_pb2
from tink.proto import tink_pb2
import tink
from tink import aead
from tink import core
from tink import hybrid


def setUpModule():
  hybrid.register()


class HybridKeyManagerTest(parameterized.TestCase):

  def test_new_key_data(self):
    tmpl = hybrid.hybrid_key_templates.ECIES_P256_HKDF_HMAC_SHA256_AES128_GCM
    key_manager = core.Registry.key_manager(tmpl.type_url)
    key_data = key_manager.new_key_data(tmpl)
    self.assertEqual(key_data.type_url, key_manager.key_type())
    self.assertEqual(key_data.key_material_type,
                     tink_pb2.KeyData.ASYMMETRIC_PRIVATE)
    key = ecies_aead_hkdf_pb2.EciesAeadHkdfPrivateKey.FromString(key_data.value)
    self.assertLen(key.key_value, 32)
    self.assertEqual(key.public_key.params.kem_params.curve_type,
                     common_pb2.NIST_P256)

  def test_new_keyset_handle_invalid_params_throw_exception(self):
    templates = hybrid.hybrid_key_templates
    key_template = templates.create_ecies_aead_hkdf_key_template(
        curve_type=cast(common_pb2.EllipticCurveType, 100),
        ec_point_format=common_pb2.UNCOMPRESSED,
        hash_type=common_pb2.SHA256,
        dem_key_template=aead.aead_key_templates.AES128_GCM)
    with self.assertRaises(core.TinkError):
      tink.new_keyset_handle(key_template)

  def test_new_keyset_handle_on_public_key_fails(self):
    key_format = ecies_aead_hkdf_pb2.EciesAeadHkdfKeyFormat()
    key_template = tink_pb2.KeyTemplate()
    key_template.type_url = (
        'type.googleapis.com/google.crypto.tink.EciesAeadHkdfPublicKey')
    key_template.value = key_format.SerializeToString()
    key_template.output_prefix_type = tink_pb2.TINK
    with self.assertRaises(core.TinkError):
      tink.new_keyset_handle(key_template)

  @parameterized.parameters([
      hybrid.hybrid_key_templates.ECIES_P256_HKDF_HMAC_SHA256_AES128_GCM, hybrid
      .hybrid_key_templates.ECIES_P256_HKDF_HMAC_SHA256_AES128_CTR_HMAC_SHA256
  ])
  def test_encrypt_decrypt(self, template):
    private_handle = tink.new_keyset_handle(template)
    public_handle = private_handle.public_keyset_handle()
    hybrid_enc = public_handle.primitive(hybrid.HybridEncrypt)
    ciphertext = hybrid_enc.encrypt(b'some plaintext', b'some context info')
    hybrid_dec = private_handle.primitive(hybrid.HybridDecrypt)
    self.assertEqual(hybrid_dec.decrypt(ciphertext, b'some context info'),
                     b'some plaintext')

  @parameterized.parameters([
      hybrid.hybrid_key_templates.ECIES_P256_HKDF_HMAC_SHA256_AES128_GCM, hybrid
      .hybrid_key_templates.ECIES_P256_HKDF_HMAC_SHA256_AES128_CTR_HMAC_SHA256
  ])
  def test_decrypt_fails(self, template):
    private_handle = tink.new_keyset_handle(template)
    hybrid_dec = private_handle.primitive(hybrid.HybridDecrypt)
    with self.assertRaises(core.TinkError):
      hybrid_dec.decrypt(b'bad ciphertext', b'some context info')


class HpkeKeyManagerTest(parameterized.TestCase):

  def test_new_key_data(self):
    tmpl = hybrid.hybrid_key_templates.DHKEM_X25519_HKDF_SHA256_HKDF_SHA256_AES_128_GCM
    key_manager = core.Registry.key_manager(tmpl.type_url)
    key_data = key_manager.new_key_data(tmpl)
    self.assertEqual(key_data.type_url, key_manager.key_type())
    self.assertEqual(key_data.key_material_type,
                     tink_pb2.KeyData.ASYMMETRIC_PRIVATE)
    key = hpke_pb2.HpkePrivateKey.FromString(key_data.value)
    self.assertLen(key.private_key, 32)  # HPKE 'Nsk' parameter length  = 32
    self.assertEqual(key.public_key.params.kem,
                     hpke_pb2.DHKEM_X25519_HKDF_SHA256)
    self.assertEqual(key.public_key.params.kdf, hpke_pb2.HKDF_SHA256)
    self.assertEqual(key.public_key.params.aead, hpke_pb2.AES_128_GCM)

  def test_new_keyset_handle_invalid_kem_raises_exception(self):
    templates = hybrid.hybrid_key_templates
    key_template = templates._create_hpke_key_template(
        hpke_kem=hpke_pb2.KEM_UNKNOWN,
        hpke_kdf=hpke_pb2.HKDF_SHA256,
        hpke_aead=hpke_pb2.AES_128_GCM,
        output_prefix_type=tink_pb2.TINK)
    with self.assertRaises(core.TinkError):
      tink.new_keyset_handle(key_template)

  def test_new_keyset_handle_invalid_kdf_raises_exception(self):
    templates = hybrid.hybrid_key_templates
    key_template = templates._create_hpke_key_template(
        hpke_kem=hpke_pb2.DHKEM_X25519_HKDF_SHA256,
        hpke_kdf=hpke_pb2.KDF_UNKNOWN,
        hpke_aead=hpke_pb2.AES_128_GCM,
        output_prefix_type=tink_pb2.TINK)
    with self.assertRaises(core.TinkError):
      tink.new_keyset_handle(key_template)

  def test_new_keyset_handle_invalid_aead_raises_exception(self):
    templates = hybrid.hybrid_key_templates
    key_template = templates._create_hpke_key_template(
        hpke_kem=hpke_pb2.DHKEM_X25519_HKDF_SHA256,
        hpke_kdf=hpke_pb2.HKDF_SHA256,
        hpke_aead=hpke_pb2.AEAD_UNKNOWN,
        output_prefix_type=tink_pb2.TINK)
    with self.assertRaises(core.TinkError):
      tink.new_keyset_handle(key_template)

  def test_new_keyset_handle_on_public_key_fails(self):
    key_format = hpke_pb2.HpkeKeyFormat()
    key_template = tink_pb2.KeyTemplate()
    key_template.type_url = (
        'type.googleapis.com/google.crypto.tink.HpkePublicKey')
    key_template.value = key_format.SerializeToString()
    key_template.output_prefix_type = tink_pb2.TINK
    with self.assertRaises(core.TinkError):
      tink.new_keyset_handle(key_template)

  @parameterized.parameters([
      hybrid.hybrid_key_templates
      .DHKEM_X25519_HKDF_SHA256_HKDF_SHA256_AES_128_GCM,
      hybrid.hybrid_key_templates
      .DHKEM_X25519_HKDF_SHA256_HKDF_SHA256_AES_128_GCM_RAW, hybrid
      .hybrid_key_templates.DHKEM_X25519_HKDF_SHA256_HKDF_SHA256_AES_256_GCM,
      hybrid.hybrid_key_templates
      .DHKEM_X25519_HKDF_SHA256_HKDF_SHA256_AES_256_GCM_RAW,
      hybrid.hybrid_key_templates
      .DHKEM_X25519_HKDF_SHA256_HKDF_SHA256_CHACHA20_POLY1305,
      hybrid.hybrid_key_templates
      .DHKEM_X25519_HKDF_SHA256_HKDF_SHA256_CHACHA20_POLY1305_RAW
  ])
  def test_encrypt_decrypt(self, template):
    private_handle = tink.new_keyset_handle(template)
    public_handle = private_handle.public_keyset_handle()
    hybrid_encrypt = public_handle.primitive(hybrid.HybridEncrypt)
    ciphertext = hybrid_encrypt.encrypt(b'some plaintext', b'some context info')
    hybrid_decrypt = private_handle.primitive(hybrid.HybridDecrypt)
    self.assertEqual(
        hybrid_decrypt.decrypt(ciphertext, b'some context info'),
        b'some plaintext')

  @parameterized.parameters([
      hybrid.hybrid_key_templates
      .DHKEM_X25519_HKDF_SHA256_HKDF_SHA256_AES_128_GCM,
      hybrid.hybrid_key_templates
      .DHKEM_X25519_HKDF_SHA256_HKDF_SHA256_AES_128_GCM_RAW, hybrid
      .hybrid_key_templates.DHKEM_X25519_HKDF_SHA256_HKDF_SHA256_AES_256_GCM,
      hybrid.hybrid_key_templates
      .DHKEM_X25519_HKDF_SHA256_HKDF_SHA256_AES_256_GCM_RAW,
      hybrid.hybrid_key_templates
      .DHKEM_X25519_HKDF_SHA256_HKDF_SHA256_CHACHA20_POLY1305,
      hybrid.hybrid_key_templates
      .DHKEM_X25519_HKDF_SHA256_HKDF_SHA256_CHACHA20_POLY1305_RAW
  ])
  def test_decrypt_fails(self, template):
    private_handle = tink.new_keyset_handle(template)
    hybrid_decrypt = private_handle.primitive(hybrid.HybridDecrypt)
    with self.assertRaises(core.TinkError):
      hybrid_decrypt.decrypt(b'bad ciphertext', b'some context info')

if __name__ == '__main__':
  absltest.main()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Python wrapper of the wrapped C++ Hybrid En- and Decryption key manager."""

from tink import core
from tink.cc.pybind import tink_bindings
from tink.hybrid import _hybrid_decrypt
from tink.hybrid import _hybrid_encrypt
from tink.hybrid import _hybrid_wrapper


class _HybridDecryptCcToPyWrapper(_hybrid_decrypt.HybridDecrypt):
  """Transforms C++ HybridDecrypt primitive into a Python primitive."""

  def __init__(self, cc_primitive: tink_bindings.HybridDecrypt):
    self._hybrid_decrypt = cc_primitive

  @core.use_tink_errors
  def decrypt(self, ciphertext: bytes, context_info: bytes) -> bytes:
    return self._hybrid_decrypt.decrypt(ciphertext, context_info)


class _HybridEncryptCcToPyWrapper(_hybrid_encrypt.HybridEncrypt):
  """Transforms C++ HybridEncrypt primitive into a Python primitive."""

  def __init__(self, cc_primitive: tink_bindings.HybridEncrypt):
    self._hybrid_encrypt = cc_primitive

  @core.use_tink_errors
  def encrypt(self, plaintext: bytes, context_info: bytes) -> bytes:
    return self._hybrid_encrypt.encrypt(plaintext, context_info)


def register():
  """Registers all Hybrid key managers and wrapper in the Python Registry."""
  tink_bindings.register()
  tink_bindings.register_hpke()

  # Register primitive wrappers.
  core.Registry.register_primitive_wrapper(
      _hybrid_wrapper.HybridDecryptWrapper())
  core.Registry.register_primitive_wrapper(
      _hybrid_wrapper.HybridEncryptWrapper())

  # Register ECIES-AEAD-HKDF key managers.
  decrypt_type_url = (
      'type.googleapis.com/google.crypto.tink.EciesAeadHkdfPrivateKey')
  decrypt_key_manager = core.PrivateKeyManagerCcToPyWrapper(
      tink_bindings.HybridDecryptKeyManager.from_cc_registry(decrypt_type_url),
      _hybrid_decrypt.HybridDecrypt, _HybridDecryptCcToPyWrapper)
  core.Registry.register_key_manager(decrypt_key_manager, new_key_allowed=True)

  encrypt_type_url = (
      'type.googleapis.com/google.crypto.tink.EciesAeadHkdfPublicKey')
  encrypt_key_manager = core.KeyManagerCcToPyWrapper(
      tink_bindings.HybridEncryptKeyManager.from_cc_registry(encrypt_type_url),
      _hybrid_encrypt.HybridEncrypt, _HybridEncryptCcToPyWrapper)
  core.Registry.register_key_manager(encrypt_key_manager, new_key_allowed=True)

  # Register HPKE key managers.
  hpke_decrypt_type_url = (
      'type.googleapis.com/google.crypto.tink.HpkePrivateKey')
  hpke_decrypt_key_manager = core.PrivateKeyManagerCcToPyWrapper(
      tink_bindings.HybridDecryptKeyManager.from_cc_registry(
          hpke_decrypt_type_url), _hybrid_decrypt.HybridDecrypt,
      _HybridDecryptCcToPyWrapper)
  core.Registry.register_key_manager(
      hpke_decrypt_key_manager, new_key_allowed=True)

  hpke_encrypt_type_url = (
      'type.googleapis.com/google.crypto.tink.HpkePublicKey')
  hpke_encrypt_key_manager = core.KeyManagerCcToPyWrapper(
      tink_bindings.HybridEncryptKeyManager.from_cc_registry(
          hpke_encrypt_type_url), _hybrid_encrypt.HybridEncrypt,
      _HybridEncryptCcToPyWrapper)
  core.Registry.register_key_manager(
      hpke_encrypt_key_manager, new_key_allowed=True)



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.hybrid_key_templates."""

from absl.testing import absltest
from absl.testing import parameterized
from tink.proto import common_pb2
from tink.proto import ecies_aead_hkdf_pb2
from tink.proto import hpke_pb2
from tink.proto import tink_pb2
from tink import aead
from tink import hybrid


class HybridKeyTemplatesTest(parameterized.TestCase):

  def test_create_aes_eax_key_template(self):
    # Intentionally using 'weird' or invalid values for parameters,
    # to test that the function correctly puts them in the resulting template.
    template = None
    with self.assertWarns(DeprecationWarning):
      template = (
          hybrid.hybrid_key_templates.create_ecies_aead_hkdf_key_template(
              curve_type=common_pb2.NIST_P521,
              ec_point_format=common_pb2.DO_NOT_USE_CRUNCHY_UNCOMPRESSED,
              hash_type=common_pb2.SHA1,
              dem_key_template=aead.aead_key_templates.AES256_EAX))
    self.assertEqual(
        'type.googleapis.com/google.crypto.tink.EciesAeadHkdfPrivateKey',
        template.type_url)
    self.assertEqual(tink_pb2.TINK, template.output_prefix_type)
    key_format = ecies_aead_hkdf_pb2.EciesAeadHkdfKeyFormat.FromString(
        template.value)
    self.assertEqual(key_format.params.kem_params.curve_type,
                     common_pb2.NIST_P521)
    self.assertEqual(key_format.params.kem_params.hkdf_hash_type,
                     common_pb2.SHA1)
    self.assertEqual(key_format.params.ec_point_format,
                     common_pb2.DO_NOT_USE_CRUNCHY_UNCOMPRESSED)
    self.assertEqual(key_format.params.dem_params.aead_dem,
                     aead.aead_key_templates.AES256_EAX)

  def test_create_hpke_key_template(self):
    template = hybrid.hybrid_key_templates._create_hpke_key_template(
        hpke_kem=hpke_pb2.DHKEM_X25519_HKDF_SHA256,
        hpke_kdf=hpke_pb2.HKDF_SHA256,
        hpke_aead=hpke_pb2.AES_128_GCM,
        output_prefix_type=tink_pb2.TINK)
    self.assertEqual('type.googleapis.com/google.crypto.tink.HpkePrivateKey',
                     template.type_url)
    self.assertEqual(tink_pb2.TINK, template.output_prefix_type)
    key_format = hpke_pb2.HpkeKeyFormat.FromString(template.value)
    self.assertEqual(key_format.params.kem, hpke_pb2.DHKEM_X25519_HKDF_SHA256)
    self.assertEqual(key_format.params.kdf, hpke_pb2.HKDF_SHA256)
    self.assertEqual(key_format.params.aead, hpke_pb2.AES_128_GCM)


if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.aead_wrapper."""

from absl.testing import absltest
import tink
from tink import prf
from tink.testing import keyset_builder


TEMPLATE = prf.prf_key_templates.HMAC_SHA256


def setUpModule():
  prf.register()


class PrfSetWrapperTest(absltest.TestCase):

  def test_wrapped_output_is_equal(self):
    keyset_handle = tink.new_keyset_handle(TEMPLATE)
    primitive = keyset_handle.primitive(prf.PrfSet)
    output = primitive.primary().compute(b'input', output_length=31)
    key_id = primitive.primary_id()
    prfs = primitive.all()
    self.assertLen(prfs, 1)
    self.assertEqual(prfs[key_id].compute(b'input', output_length=31), output)

  def test_invalid_length_fails(self):
    keyset_handle = tink.new_keyset_handle(TEMPLATE)
    primitive = keyset_handle.primitive(prf.PrfSet)
    with self.assertRaises(tink.TinkError):
      _ = primitive.primary().compute(b'input', output_length=1234567)
    prfs = primitive.all()
    self.assertLen(prfs, 1)
    with self.assertRaises(tink.TinkError):
      _ = prfs[primitive.primary_id()].compute(b'input', output_length=1234567)

  def test_wrap_three_with_one_disabled(self):
    builder = keyset_builder.new_keyset_builder()
    id1 = builder.add_new_key(TEMPLATE)
    id2 = builder.add_new_key(TEMPLATE)
    disabled_id = builder.add_new_key(TEMPLATE)
    builder.disable_key(disabled_id)
    builder.set_primary_key(id1)
    prf_set1 = builder.keyset_handle().primitive(prf.PrfSet)
    builder.set_primary_key(id2)
    prf_set2 = builder.keyset_handle().primitive(prf.PrfSet)
    self.assertNotEqual(id1, id2)
    self.assertEqual(prf_set1.primary_id(), id1)
    self.assertEqual(prf_set2.primary_id(), id2)

    output1 = prf_set1.primary().compute(b'input', output_length=31)
    output2 = prf_set2.primary().compute(b'input', output_length=31)
    self.assertNotEqual(output1, output2)
    prfs = prf_set1.all()
    self.assertLen(prfs, 2)
    self.assertEqual(prfs[id1].compute(b'input', output_length=31),
                     output1)
    self.assertEqual(prfs[id2].compute(b'input', output_length=31),
                     output2)


if __name__ == '__main__':
  absltest.main()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Core package."""

from tink.proto import tink_pb2
from tink.core import _crypto_format
from tink.core import _key_manager
from tink.core import _primitive_set
from tink.core import _primitive_wrapper
from tink.core import _registry
from tink.core import _tink_error


KeyManager = _key_manager.KeyManager
PrivateKeyManager = _key_manager.PrivateKeyManager

KeyManagerCcToPyWrapper = _key_manager.KeyManagerCcToPyWrapper
PrivateKeyManagerCcToPyWrapper = _key_manager.PrivateKeyManagerCcToPyWrapper

Registry = _registry.Registry

TinkError = _tink_error.TinkError
use_tink_errors = _tink_error.use_tink_errors

new_primitive_set = _primitive_set.new_primitive_set
PrimitiveSet = _primitive_set.PrimitiveSet
PrimitiveWrapper = _primitive_wrapper.PrimitiveWrapper

crypto_format = _crypto_format


class KeyAccess:
  """Base class for access tokens for Tink Keys."""
  pass



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Deterministic AEAD wrapper."""

from typing import Optional, Type

from tink import _monitoring
from tink import core
from tink.daead import _deterministic_aead


class _WrappedDeterministicAead(_deterministic_aead.DeterministicAead):
  """Implements DeterministicAead for a set of DeterministicAead primitives."""

  def __init__(
      self,
      pset: core.PrimitiveSet,
      encryption_monitor: Optional[_monitoring.KeyUsageMonitor] = None,
      decryption_monitor: Optional[_monitoring.KeyUsageMonitor] = None,
  ):
    self._primitive_set = pset
    self._encryption_monitor = encryption_monitor
    self._decryption_monitor = decryption_monitor

  def encrypt_deterministically(
      self, plaintext: bytes, associated_data: bytes
  ) -> bytes:
    primary = self._primitive_set.primary()
    result = primary.identifier + primary.primitive.encrypt_deterministically(
        plaintext, associated_data
    )

    if self._encryption_monitor:
      self._encryption_monitor.log(primary.key_id, len(plaintext))

    return result

  def decrypt_deterministically(
      self, ciphertext: bytes, associated_data: bytes
  ) -> bytes:
    if len(ciphertext) > core.crypto_format.NON_RAW_PREFIX_SIZE:
      prefix = ciphertext[: core.crypto_format.NON_RAW_PREFIX_SIZE]
      ciphertext_no_prefix = ciphertext[
          core.crypto_format.NON_RAW_PREFIX_SIZE :
      ]
      for entry in self._primitive_set.primitive_from_identifier(prefix):
        try:
          result = entry.primitive.decrypt_deterministically(
              ciphertext_no_prefix, associated_data
          )

          if self._decryption_monitor:
            self._decryption_monitor.log(
                entry.key_id, len(ciphertext_no_prefix)
            )

          return result
        except core.TinkError:
          pass
    # Let's try all RAW keys.
    for entry in self._primitive_set.raw_primitives():
      try:
        result = entry.primitive.decrypt_deterministically(
            ciphertext, associated_data
        )

        if self._decryption_monitor:
          self._decryption_monitor.log(entry.key_id, len(ciphertext))

        return result
      except core.TinkError:
        pass

    # nothing works.
    if self._decryption_monitor:
      self._decryption_monitor.log_failure()

    raise core.TinkError('Decryption failed.')


class DeterministicAeadWrapper(
    core.PrimitiveWrapper[
        _deterministic_aead.DeterministicAead,
        _deterministic_aead.DeterministicAead,
    ]
):
  """DeterministicAeadWrapper is a PrimitiveWrapper for DeterministicAead.

  The created primitive works with a keyset (rather than a single key). To
  encrypt a plaintext, it uses the primary key in the keyset, and prepends to
  the ciphertext a certain prefix associated with the primary key. To decrypt,
  the primitive uses the prefix of the ciphertext to efficiently select the
  right key in the set. If the keys associated with the prefix do not work, the
  primitive tries all keys with OutputPrefixType RAW.
  """

  def wrap(
      self, pset: core.PrimitiveSet
  ) -> _deterministic_aead.DeterministicAead:
    return _WrappedDeterministicAead(pset)

  def primitive_class(self) -> Type[_deterministic_aead.DeterministicAead]:
    return _deterministic_aead.DeterministicAead

  def input_primitive_class(
      self,
  ) -> Type[_deterministic_aead.DeterministicAead]:
    return _deterministic_aead.DeterministicAead

  def _wrap_with_monitoring_info(
      self,
      pset: core.PrimitiveSet,
      monitoring_keyset_info: _monitoring.MonitoringKeySetInfo,
  ) -> _deterministic_aead.DeterministicAead:
    encryption_key_usage_monitor = _monitoring.get_key_usage_monitor_or_none(
        _monitoring.MonitoringContext(
            primitive='daead',
            api_function='encrypt',
            keyset_info=monitoring_keyset_info,
        )
    )
    decryption_key_usage_monitor = _monitoring.get_key_usage_monitor_or_none(
        _monitoring.MonitoringContext(
            primitive='daead',
            api_function='decrypt',
            keyset_info=monitoring_keyset_info,
        )
    )
    return _WrappedDeterministicAead(
        pset, encryption_key_usage_monitor, decryption_key_usage_monitor
    )



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Basic interface for wrapping a primitive."""

import abc
from typing import Generic, Optional, Type, TypeVar

from tink import _monitoring
from tink.core import _primitive_set


B = TypeVar('B')
P = TypeVar('P')


class PrimitiveWrapper(Generic[B, P], metaclass=abc.ABCMeta):
  """Basic interface for wrapping a primitive.

  A PrimitiveSet can be wrapped by a single primitive in order to fulfill a
  cryptographic task. This is done by the PrimitiveWrapper. Whenever a new
  primitive type is added to Tink, the user should define a new PrimitiveWrapper
  and register it by calling registry.registerPrimitiveWrapper().

  The primitive of type B is wrapped into a primitive of type P. In most cases,
  B and P are the same.
  """

  def __init__(self, annotations: Optional[_monitoring.Annotations] = None):
    self._annotations = annotations

  @abc.abstractmethod
  def wrap(self, pset: _primitive_set.PrimitiveSet) -> P:
    """Wraps a PrimitiveSet and returns a single primitive instance."""
    raise NotImplementedError()

  @abc.abstractmethod
  def primitive_class(self) -> Type[P]:
    """Returns the class of the primitive produced by the wrapper."""
    raise NotImplementedError()

  def input_primitive_class(self) -> Type[B]:
    """Returns the class of the primitive that gets wrapped."""
    raise NotImplementedError()

  def _wrap_with_monitoring_info(
      self,
      pset: _primitive_set.PrimitiveSet,
      monitoring_keyset_info: _monitoring.MonitoringKeySetInfo,
  ) -> P:
    """Same as `wrap`, but passes along monitoring annotations.

       By default it does nothing more than `wrap`. Subclasses can override
    this method to handle monitoring annotations.

    Args:
      pset: The PrimitiveSet to wrap.
      monitoring_keyset_info: The monitoring information to pass along.

    Returns:
      The wrapped primitive.
    """
    if monitoring_keyset_info.get_annotations():
      raise NotImplementedError(
          'Support for monitoring is not implemented for this'
          ' primitive'
      )
    # Monitoring is optional and thus ignored by default.
    del monitoring_keyset_info
    return self.wrap(pset)



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Python primitive set wrapper for the JwtMac primitive."""

from typing import Type

from tink.proto import tink_pb2
from tink import core
from tink.jwt import _jwt_error
from tink.jwt import _jwt_format
from tink.jwt import _jwt_public_key_sign
from tink.jwt import _jwt_public_key_verify
from tink.jwt import _jwt_validator
from tink.jwt import _raw_jwt
from tink.jwt import _verified_jwt


class _WrappedJwtPublicKeySign(_jwt_public_key_sign.JwtPublicKeySign):
  """A wrapped JwtPublicKeySign."""

  def __init__(self, pset: core.PrimitiveSet):
    self._primitive_set = pset

  def sign_and_encode(self, raw_jwt: _raw_jwt.RawJwt) -> str:
    primary = self._primitive_set.primary()
    kid = _jwt_format.get_kid(primary.key_id, primary.output_prefix_type)
    return primary.primitive.sign_and_encode_with_kid(raw_jwt, kid)


class _WrappedJwtPublicKeyVerify(_jwt_public_key_verify.JwtPublicKeyVerify):
  """A wrapped JwtPublicKeyVerify."""

  def __init__(self, pset: core.PrimitiveSet):
    self._primitive_set = pset

  def verify_and_decode(
      self, compact: str,
      validator: _jwt_validator.JwtValidator) -> _verified_jwt.VerifiedJwt:
    interesting_error = None
    for entries in self._primitive_set.all():
      for entry in entries:
        try:
          kid = _jwt_format.get_kid(entry.key_id, entry.output_prefix_type)
          return entry.primitive.verify_and_decode_with_kid(
              compact, validator, kid)
        except core.TinkError as e:
          if isinstance(e, _jwt_error.JwtInvalidError):
            interesting_error = e
          pass
    if interesting_error:
      raise interesting_error
    raise core.TinkError('invalid signature')


def _validate_primitive_set(pset: core.PrimitiveSet):
  for entries in pset.all():
    for entry in entries:
      if (entry.output_prefix_type != tink_pb2.RAW and
          entry.output_prefix_type != tink_pb2.TINK):
        raise core.TinkError('unsupported OutputPrefixType')


class _JwtPublicKeySignWrapper(
    core.PrimitiveWrapper[_jwt_public_key_sign.JwtPublicKeySignInternal,
                          _jwt_public_key_sign.JwtPublicKeySign]):
  """A wrapper for JwtPublicKeySign."""

  def wrap(self,
           pset: core.PrimitiveSet) -> _jwt_public_key_sign.JwtPublicKeySign:
    _validate_primitive_set(pset)
    return _WrappedJwtPublicKeySign(pset)

  def primitive_class(self) -> Type[_jwt_public_key_sign.JwtPublicKeySign]:
    return _jwt_public_key_sign.JwtPublicKeySign

  def input_primitive_class(
      self) -> Type[_jwt_public_key_sign.JwtPublicKeySignInternal]:
    return _jwt_public_key_sign.JwtPublicKeySignInternal


class _JwtPublicKeyVerifyWrapper(
    core.PrimitiveWrapper[_jwt_public_key_verify.JwtPublicKeyVerifyInternal,
                          _jwt_public_key_verify.JwtPublicKeyVerify]):
  """A wrapper for JwtPublicKeyVerify."""

  def wrap(
      self,
      pset: core.PrimitiveSet) -> _jwt_public_key_verify.JwtPublicKeyVerify:
    _validate_primitive_set(pset)
    return _WrappedJwtPublicKeyVerify(pset)

  def primitive_class(self) -> Type[_jwt_public_key_verify.JwtPublicKeyVerify]:
    return _jwt_public_key_verify.JwtPublicKeyVerify

  def input_primitive_class(
      self) -> Type[_jwt_public_key_verify.JwtPublicKeyVerifyInternal]:
    return _jwt_public_key_verify.JwtPublicKeyVerifyInternal


def register():
  core.Registry.register_primitive_wrapper(_JwtPublicKeySignWrapper())
  core.Registry.register_primitive_wrapper(_JwtPublicKeyVerifyWrapper())



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.integration.gcp_kms_aead."""

import os
from unittest import mock

from absl.testing import absltest
from google.cloud import kms_v1
from google.oauth2 import service_account

import tink
from tink import aead
from tink.integration import gcpkms
from tink.testing import helper


CREDENTIAL_PATH = os.path.join(helper.tink_py_testdata_path(),
                               'gcp/credential.json')
KEY_URI = 'gcp-kms://projects/tink-test-infrastructure/locations/global/keyRings/unit-and-integration-testing/cryptoKeys/aead-key'
KEY_VERSION_URI = KEY_URI + '/cryptoKeyVersions/1'
LOCAL_KEY_URI = 'gcp-kms://projects/tink-test-infrastructure/locations/europe-west1/keyRings/unit-and-integration-test/cryptoKeys/aead-key'
BAD_KEY_URI = 'aws-kms://arn:aws:kms:us-east-2:235739564943:key/3ee50705-5a82-4f5b-9753-05c4f473922f'

KEY2_URI = 'gcp-kms://projects/tink-test-infrastructure/locations/global/keyRings/unit-and-integration-testing/cryptoKeys/aead2-key'

if 'TEST_SRCDIR' in os.environ:
  # Set root certificates for gRPC in Bazel Test which are needed on MacOS
  os.environ['GRPC_DEFAULT_SSL_ROOTS_FILE_PATH'] = os.path.join(
      os.environ['TEST_SRCDIR'], 'google_root_pem/file/downloaded')


def setUpModule():
  aead.register()


class GcpKmsAeadTest(absltest.TestCase):

  def test_bound_to_key_uri_encrypt_decrypt(self):
    with gcpkms.GcpKmsClient(KEY_URI, CREDENTIAL_PATH) as gcp_client:
      self.assertTrue(gcp_client.does_support(KEY_URI))
      self.assertFalse(gcp_client.does_support(KEY2_URI))
      self.assertFalse(gcp_client.does_support(BAD_KEY_URI))

      gcp_aead = gcp_client.get_aead(KEY_URI)

      plaintext = b'plaintext'
      associated_data = b'associated_data'
      ciphertext = gcp_aead.encrypt(plaintext, associated_data)
      self.assertEqual(plaintext, gcp_aead.decrypt(ciphertext, associated_data))

      ciphertext = gcp_aead.encrypt(plaintext, b'')
      self.assertEqual(plaintext, gcp_aead.decrypt(ciphertext, b''))

      with self.assertRaises(tink.TinkError):
        gcp_client.get_aead(KEY2_URI)

  def test_not_bound_to_key_uri_encrypt_decrypt(self):
    with gcpkms.GcpKmsClient(None, CREDENTIAL_PATH) as gcp_client:
      self.assertTrue(gcp_client.does_support(KEY_URI))
      self.assertTrue(gcp_client.does_support(KEY2_URI))
      self.assertFalse(gcp_client.does_support(BAD_KEY_URI))

      plaintext = b'plaintext'
      associated_data = b'associated_data'

      gcp_aead = gcp_client.get_aead(KEY_URI)
      ciphertext = gcp_aead.encrypt(plaintext, associated_data)
      self.assertEqual(plaintext, gcp_aead.decrypt(ciphertext, associated_data))

      gcp_aead2 = gcp_client.get_aead(KEY_URI)
      ciphertext2 = gcp_aead2.encrypt(plaintext, associated_data)
      self.assertEqual(
          plaintext, gcp_aead2.decrypt(ciphertext2, associated_data)
      )

  def test_decrypt_with_wrong_ad_fails(self):
    with gcpkms.GcpKmsClient(KEY_URI, CREDENTIAL_PATH) as gcp_client:
      gcp_aead = gcp_client.get_aead(KEY_URI)

      ciphertext = gcp_aead.encrypt(b'plaintext', b'associated_data')
      with self.assertRaises(tink.TinkError):
        gcp_aead.decrypt(ciphertext, b'wrong_associated_data')

  def test_decrypt_with_wrong_key_fails(self):
    with gcpkms.GcpKmsClient(None, CREDENTIAL_PATH) as gcp_client:
      gcp_aead1 = gcp_client.get_aead(KEY_URI)
      gcp_aead2 = gcp_client.get_aead(KEY2_URI)

      ciphertext1 = gcp_aead1.encrypt(b'plaintext', b'associated_data')
      ciphertext2 = gcp_aead2.encrypt(b'plaintext', b'associated_data')

      # First, verify that both key URIs work.
      self.assertEqual(
          b'plaintext', gcp_aead1.decrypt(ciphertext1, b'associated_data')
      )
      self.assertEqual(
          b'plaintext', gcp_aead2.decrypt(ciphertext2, b'associated_data')
      )

      with self.assertRaises(tink.TinkError):
        gcp_aead2.decrypt(ciphertext1, b'associated_data')
      with self.assertRaises(tink.TinkError):
        gcp_aead1.decrypt(ciphertext2, b'associated_data')

  def test_encrypt_decrypt_localized_uri(self):
    with gcpkms.GcpKmsClient(LOCAL_KEY_URI, CREDENTIAL_PATH) as gcp_client:
      gcp_aead = gcp_client.get_aead(LOCAL_KEY_URI)

      plaintext = b'helloworld'
      ciphertext = gcp_aead.encrypt(plaintext, b'')
      self.assertEqual(plaintext, gcp_aead.decrypt(ciphertext, b''))

      plaintext = b'hello'
      associated_data = b'world'
      ciphertext = gcp_aead.encrypt(plaintext, associated_data)
      self.assertEqual(
          plaintext, gcp_aead.decrypt(ciphertext, associated_data)
      )

  def test_encrypt_with_bad_uri(self):
    with self.assertRaises(tink.TinkError):
      with gcpkms.GcpKmsClient(KEY_URI, CREDENTIAL_PATH) as gcp_client:
        gcp_client.get_aead(BAD_KEY_URI)

  def test_corrupted_ciphertext(self):
    with gcpkms.GcpKmsClient(KEY_URI, CREDENTIAL_PATH) as gcp_client:
      gcp_aead = gcp_client.get_aead(KEY_URI)

      plaintext = b'helloworld'
      ciphertext = gcp_aead.encrypt(plaintext, b'')
      self.assertEqual(plaintext, gcp_aead.decrypt(ciphertext, b''))

      # Corrupt each byte once and check that decryption fails
      # NOTE: Only starting at 4th byte here, as the 3rd byte is malleable
      #      (see b/146633745).
      for byte_idx in range(3, len(ciphertext)):
        tmp_ciphertext = list(ciphertext)
        tmp_ciphertext[byte_idx] ^= 1
        corrupted_ciphertext = bytes(tmp_ciphertext)
        with self.assertRaises(tink.TinkError):
          gcp_aead.decrypt(corrupted_ciphertext, b'')

  def test_init_with_credentials_and_credentials_path_fails(self):
    credentials = mock.Mock()
    with self.assertRaises(tink.TinkError):
      gcpkms.GcpKmsClient(KEY_URI, CREDENTIAL_PATH, credentials=credentials)

  def test_init_with_credentials_no_credentials_path(self):
    mock_client = mock.Mock()
    with mock.patch.object(
        kms_v1, 'KeyManagementServiceClient', return_value=mock_client
    ) as mock_kms_client_constructor:
      credentials = mock.Mock()
      with gcpkms.GcpKmsClient(KEY_URI, credentials=credentials):
        mock_kms_client_constructor.assert_called_with(credentials=credentials)

  def test_init_no_credentials_with_credentials_path(self):
    mock_client = mock.Mock()
    with mock.patch.object(
        kms_v1, 'KeyManagementServiceClient', return_value=mock_client
    ) as mock_kms_client_constructor:
      mock_credentials = mock.Mock()
      with mock.patch.object(
          service_account.Credentials,
          'from_service_account_file',
          return_value=mock_credentials,
      ):
        with gcpkms.GcpKmsClient(KEY_URI, CREDENTIAL_PATH):
          mock_kms_client_constructor.assert_called_with(
              credentials=mock_credentials
          )

  def test_init_no_credentials_no_credentials_path(self):
    mock_client = mock.Mock()
    with mock.patch.object(
        kms_v1, 'KeyManagementServiceClient', return_value=mock_client
    ) as mock_kms_client_constructor:
      with gcpkms.GcpKmsClient(KEY_URI):
        mock_kms_client_constructor.assert_called()

  def test_encrypt_decrypt_with_key_version_uri(self):
    with gcpkms.GcpKmsClient(KEY_VERSION_URI, CREDENTIAL_PATH) as gcp_client:
      self.assertTrue(gcp_client.does_support(KEY_VERSION_URI))

      gcp_aead = gcp_client.get_aead(KEY_VERSION_URI)

      plaintext = b'plaintext'
      associated_data = b'associated_data'
      # Encryption is supported.
      ciphertext = gcp_aead.encrypt(plaintext, associated_data)
      # Decrpytion is not.
      with self.assertRaises(tink.TinkError):
        gcp_aead.decrypt(ciphertext, associated_data)


if __name__ == '__main__':
  absltest.main()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink._keyset_writer."""

import io

from typing import cast
from absl.testing import absltest

from tink.proto import tink_pb2
import tink
from tink import core


def example_keyset() -> tink_pb2.Keyset:
  keyset = tink_pb2.Keyset()
  keyset.primary_key_id = 42
  key = keyset.key.add()
  key.key_data.type_url = u'type.googleapis.com/google.crypto.tink.AesGcmKey'
  key.key_data.key_material_type = tink_pb2.KeyData.SYMMETRIC
  key.key_data.value = b'GhCS/1+ejWpx68NfGt6ziYHd'
  key.output_prefix_type = tink_pb2.TINK
  key.key_id = 42
  key.status = tink_pb2.ENABLED
  return keyset


def example_encrypted_keyset() -> tink_pb2.EncryptedKeyset:
  encrypted_keyset = tink_pb2.EncryptedKeyset()
  encrypted_keyset.encrypted_keyset = b'c29tZSBjaXBoZXJ0ZXh0IHdpdGgga2V5c2V0'
  encrypted_keyset.keyset_info.primary_key_id = 42
  key_info = encrypted_keyset.keyset_info.key_info.add()
  key_info.type_url = 'type.googleapis.com/google.crypto.tink.AesGcmKey'
  key_info.output_prefix_type = tink_pb2.TINK
  key_info.key_id = 42
  key_info.status = tink_pb2.ENABLED
  return encrypted_keyset


class JsonKeysetWriterTest(absltest.TestCase):

  def test_write_read(self):
    keyset = example_keyset()
    stream = io.StringIO()
    writer = tink.JsonKeysetWriter(stream)
    writer.write(keyset)
    reader = tink.JsonKeysetReader(stream.getvalue())
    self.assertEqual(keyset, reader.read())

  def test_write_encrypted_read_encrypted(self):
    encrypted_keyset = example_encrypted_keyset()
    stream = io.StringIO()
    writer = tink.JsonKeysetWriter(stream)
    writer.write_encrypted(encrypted_keyset)
    reader = tink.JsonKeysetReader(stream.getvalue())
    self.assertEqual(
        encrypted_keyset.encrypted_keyset,
        reader.read_encrypted().encrypted_keyset,
    )

  def test_write_read_with_unicode_chars(self):
    keyset = tink_pb2.Keyset()
    key = keyset.key.add()
    key.key_data.type_url = (
        u'\xe3\x82\xb3\xe3\x83\xb3\xe3\x83\x8b\xe3\x83\x81\xe3\x83\x8f')
    stream = io.StringIO()
    writer = tink.JsonKeysetWriter(stream)
    writer.write(keyset)
    reader = tink.JsonKeysetReader(stream.getvalue())
    self.assertEqual(keyset, reader.read())

  def test_write_invalid_fails(self):
    stream = io.StringIO()
    writer = tink.JsonKeysetWriter(stream)
    invalid_keyset = cast(tink_pb2.Keyset, example_encrypted_keyset())
    with self.assertRaises(core.TinkError):
      writer.write(invalid_keyset)

  def test_write_encrypted_invalid_fails(self):
    stream = io.StringIO()
    writer = tink.JsonKeysetWriter(stream)
    invalid_encrypted_keyset = cast(tink_pb2.EncryptedKeyset, example_keyset())
    with self.assertRaises(core.TinkError):
      writer.write_encrypted(invalid_encrypted_keyset)


class BinaryKeysetReaderTest(absltest.TestCase):

  def test_write_read(self):
    keyset = example_keyset()
    stream = io.BytesIO()
    writer = tink.BinaryKeysetWriter(stream)
    writer.write(keyset)
    reader = tink.BinaryKeysetReader(stream.getvalue())
    self.assertEqual(keyset, reader.read())

  def test_write_encrypted_read_encrypted(self):
    encrypted_keyset = example_encrypted_keyset()
    stream = io.BytesIO()
    writer = tink.BinaryKeysetWriter(stream)
    writer.write_encrypted(encrypted_keyset)
    reader = tink.BinaryKeysetReader(stream.getvalue())
    self.assertEqual(
        encrypted_keyset.encrypted_keyset,
        reader.read_encrypted().encrypted_keyset,
    )

  def test_write_invalid_fails(self):
    stream = io.BytesIO()
    writer = tink.BinaryKeysetWriter(stream)
    invalid_keyset = cast(tink_pb2.Keyset, example_encrypted_keyset())
    with self.assertRaises(core.TinkError):
      writer.write(invalid_keyset)

  def test_write_encrypted_invalid_fails(self):
    stream = io.BytesIO()
    writer = tink.BinaryKeysetWriter(stream)
    invalid_encrypted_keyset = cast(tink_pb2.EncryptedKeyset, example_keyset())
    with self.assertRaises(core.TinkError):
      writer.write_encrypted(invalid_encrypted_keyset)


if __name__ == '__main__':
  absltest.main()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Interface for PublicKeyVerify."""

import abc


class PublicKeyVerify(metaclass=abc.ABCMeta):
  """Interface for public key verifying.

  Digital Signatures provide functionality of signing data and verification of
  the signatures. They are represented by a pair of primitives (interfaces)
  'PublicKeySign' for signing of data, and 'PublicKeyVerify' for verification
  of signatures. Implementations of these interfaces are secure against
  adaptive chosen-message attacks. Signing data ensures the authenticity and
  the integrity of that data, but not its secrecy.
  """

  @abc.abstractmethod
  def verify(self, signature: bytes, data: bytes) -> bytes:
    """Verifies that signature is a digital signature for data.

    Args:
      signature: The signature bytes to be checked.
      data: The data bytes to be checked.

    Raises:
      tink.TinkError if the verification fails.
    """
    raise NotImplementedError()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink._signature_key_manager."""

from absl.testing import absltest
from absl.testing import parameterized

from tink.proto import common_pb2
from tink.proto import ecdsa_pb2
from tink.proto import tink_pb2
import tink
from tink import core
from tink import signature


def setUpModule():
  signature.register()


class PublicKeySignKeyManagerTest(parameterized.TestCase):

  def test_new_key_data_ecdsa(self):
    template = None
    with self.assertWarns(DeprecationWarning):
      template = signature.signature_key_templates.create_ecdsa_key_template(
          common_pb2.SHA256, common_pb2.NIST_P256, ecdsa_pb2.DER)
    key_manager = core.Registry.key_manager(template.type_url)
    key_data = key_manager.new_key_data(template)
    self.assertEqual(key_data.type_url, template.type_url)
    key = ecdsa_pb2.EcdsaPrivateKey()
    key.ParseFromString(key_data.value)
    public_key = key.public_key
    self.assertEqual(key.version, 0)
    self.assertEqual(public_key.version, 0)
    self.assertEqual(public_key.params.hash_type, common_pb2.SHA256)
    self.assertEqual(public_key.params.curve, common_pb2.NIST_P256)
    self.assertEqual(public_key.params.encoding, ecdsa_pb2.DER)
    self.assertLen(key.key_value, 32)

  def test_new_public_keyset_handle_fails(self):
    params = ecdsa_pb2.EcdsaParams(
        hash_type=common_pb2.SHA256,
        curve=common_pb2.NIST_P256,
        encoding=ecdsa_pb2.DER)
    key_format = ecdsa_pb2.EcdsaKeyFormat(params=params)
    template = tink_pb2.KeyTemplate()
    template.type_url = 'type.googleapis.com/google.crypto.tink.EcdsaPublicKey'
    template.value = key_format.SerializeToString()
    with self.assertRaises(core.TinkError):
      tink.new_keyset_handle(template)

  @parameterized.parameters([
      signature.signature_key_templates.ECDSA_P256,
      signature.signature_key_templates.ECDSA_P384_SHA384,
      signature.signature_key_templates.ECDSA_P384_SHA512,
      signature.signature_key_templates.ECDSA_P521,
      signature.signature_key_templates.ECDSA_P256_IEEE_P1363,
      signature.signature_key_templates.ECDSA_P384_SHA384_IEEE_P1363,
      signature.signature_key_templates.ECDSA_P521_IEEE_P1363,
      signature.signature_key_templates.ED25519,
      signature.signature_key_templates.RSA_SSA_PKCS1_3072_SHA256_F4,
      signature.signature_key_templates.RSA_SSA_PKCS1_4096_SHA512_F4,
      signature.signature_key_templates.RSA_SSA_PSS_3072_SHA256_SHA256_32_F4,
      signature.signature_key_templates.RSA_SSA_PSS_4096_SHA512_SHA512_64_F4,
  ])
  def test_sign_verify_success(self, template):
    private_handle = tink.new_keyset_handle(template)
    public_handle = private_handle.public_keyset_handle()
    verifier = public_handle.primitive(signature.PublicKeyVerify)
    signer = private_handle.primitive(signature.PublicKeySign)

    data = b'data'
    data_signature = signer.sign(data)
    verifier.verify(data_signature, data)

  @parameterized.parameters([
      signature.signature_key_templates.ECDSA_P256,
      signature.signature_key_templates.ECDSA_P384,
      signature.signature_key_templates.ECDSA_P384_SHA384,
      signature.signature_key_templates.ECDSA_P521,
      signature.signature_key_templates.ECDSA_P256_IEEE_P1363,
      signature.signature_key_templates.ECDSA_P384_IEEE_P1363,
      signature.signature_key_templates.ECDSA_P384_SHA384_IEEE_P1363,
      signature.signature_key_templates.ECDSA_P521_IEEE_P1363,
      signature.signature_key_templates.ED25519,
      signature.signature_key_templates.RSA_SSA_PKCS1_3072_SHA256_F4,
      signature.signature_key_templates.RSA_SSA_PKCS1_4096_SHA512_F4,
      signature.signature_key_templates.RSA_SSA_PSS_3072_SHA256_SHA256_32_F4,
      signature.signature_key_templates.RSA_SSA_PSS_4096_SHA512_SHA512_64_F4,
  ])
  def test_verify_wrong_fails(self, template):
    private_handle = tink.new_keyset_handle(template)
    public_handle = private_handle.public_keyset_handle()
    verifier = public_handle.primitive(signature.PublicKeyVerify)
    signer = private_handle.primitive(signature.PublicKeySign)

    with self.assertRaises(core.TinkError):
      verifier.verify(signer.sign(b'data'), b'wrongdata')

    with self.assertRaises(core.TinkError):
      verifier.verify(b'wrongsignature', b'data')


if __name__ == '__main__':
  absltest.main()



# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This module defines the Tink Proto Keyset serialization Format."""

import io

from tink import _insecure_keyset_handle
from tink import _keyset_handle
from tink import _keyset_reader
from tink import _keyset_writer
from tink import aead
from tink import core


def parse(
    serialized_keyset: str, token: core.KeyAccess
) -> _keyset_handle.KeysetHandle:
  """Parses a Tink keyset. Access is restricted with a token."""
  keyset = _keyset_reader.JsonKeysetReader(serialized_keyset).read()
  return _insecure_keyset_handle.from_proto_keyset(keyset, token)


def serialize(
    keyset_handle: _keyset_handle.KeysetHandle, token: core.KeyAccess
) -> str:
  """Serializes a Tink keyset. Access is restricted with a token."""
  output_stream = io.StringIO()
  writer = _keyset_writer.JsonKeysetWriter(output_stream)
  writer.write(_insecure_keyset_handle.to_proto_keyset(keyset_handle, token))
  return output_stream.getvalue()


def parse_without_secret(
    serialized_keyset: str,
) -> _keyset_handle.KeysetHandle:
  """Parses a Tink keyset. The keyset must not contain any secret data."""
  return _keyset_handle.read_no_secret_keyset_handle(
      _keyset_reader.JsonKeysetReader(serialized_keyset)
  )


def serialize_without_secret(
    keyset_handle: _keyset_handle.KeysetHandle,
) -> str:
  """Serializes a Tink keyset. The keyset must not contain any secret data."""
  output_stream = io.StringIO()
  writer = _keyset_writer.JsonKeysetWriter(output_stream)
  keyset_handle.write_no_secret(writer)
  return output_stream.getvalue()


def parse_encrypted(
    serialized_encrypted_keyset: str,
    keyset_encryption_aead: aead.Aead,
    associated_data: bytes,
) -> _keyset_handle.KeysetHandle:
  """Parses an encrypted Tink keyset."""
  return _keyset_handle.read_keyset_handle_with_associated_data(
      _keyset_reader.JsonKeysetReader(serialized_encrypted_keyset),
      keyset_encryption_aead,
      associated_data,
  )


def serialize_encrypted(
    keyset_handle: _keyset_handle.KeysetHandle,
    keyset_encryption_aead: aead.Aead,
    associated_data: bytes,
) -> str:
  """Serializes an encrypted Tink keyset."""
  output_stream = io.StringIO()
  writer = _keyset_writer.JsonKeysetWriter(output_stream)
  keyset_handle.write_with_associated_data(
      writer,
      keyset_encryption_aead,
      associated_data,
  )
  return output_stream.getvalue()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Pre-generated KeyTemplate for DeterministicAead.

One can use these templates to generate a new tink_pb2.Keyset with
tink_pb2.KeysetHandle. To generate a new keyset that contains a single
tink_pb2.HmacKey, one can do:
handle = keyset_handle.KeysetHandle(aead_key_templates.AES128_EAX).
"""

import warnings

from tink.proto import aes_siv_pb2
from tink.proto import tink_pb2


_AES_SIV_KEY_TYPE_URL = 'type.googleapis.com/google.crypto.tink.AesSivKey'


def _create_aes_siv_key_template(key_size: int) -> tink_pb2.KeyTemplate:
  """Creates an AES EAX KeyTemplate, and fills in its values."""
  key_format = aes_siv_pb2.AesSivKeyFormat(
      key_size=key_size,
  )
  key_template = tink_pb2.KeyTemplate(
      type_url=_AES_SIV_KEY_TYPE_URL,
      output_prefix_type=tink_pb2.TINK,
      value=key_format.SerializeToString(),
  )
  return key_template


AES256_SIV = _create_aes_siv_key_template(key_size=64)


# Deprecated. Use the predefined constant AES256_SIV instead.
def create_aes_siv_key_template(key_size: int) -> tink_pb2.KeyTemplate:
  warnings.warn('The "create_aes_siv_key_template" function is deprecated.',
                DeprecationWarning, 2)
  return _create_aes_siv_key_template(key_size)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/xchacha20_poly1305.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/xchacha20_poly1305.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#tink/proto/xchacha20_poly1305.proto\x12\x12google.crypto.tink\"-\n\x1aXChaCha20Poly1305KeyFormat\x12\x0f\n\x07version\x18\x01 \x01(\r\":\n\x14XChaCha20Poly1305Key\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x11\n\tkey_value\x18\x03 \x01(\x0c\x42\x65\n\x1c\x63om.google.crypto.tink.protoP\x01ZCgithub.com/tink-crypto/tink-go/v2/proto/xchacha20_poly1305_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.xchacha20_poly1305_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001ZCgithub.com/tink-crypto/tink-go/v2/proto/xchacha20_poly1305_go_proto'
  _globals['_XCHACHA20POLY1305KEYFORMAT']._serialized_start=59
  _globals['_XCHACHA20POLY1305KEYFORMAT']._serialized_end=104
  _globals['_XCHACHA20POLY1305KEY']._serialized_start=106
  _globals['_XCHACHA20POLY1305KEY']._serialized_end=164
# @@protoc_insertion_point(module_scope)



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Hybrid package."""

from tink.hybrid import _hybrid_decrypt
from tink.hybrid import _hybrid_encrypt
from tink.hybrid import _hybrid_key_manager
from tink.hybrid import _hybrid_key_templates as hybrid_key_templates


HybridDecrypt = _hybrid_decrypt.HybridDecrypt
HybridEncrypt = _hybrid_encrypt.HybridEncrypt
register = _hybrid_key_manager.register



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.jwt._jwt_hmac_key_manager."""

import base64
import datetime

from typing import cast, Any

from absl.testing import absltest
from absl.testing import parameterized

from tink.proto import jwt_hmac_pb2
from tink.proto import tink_pb2
import tink
from tink import jwt
from tink.cc.pybind import tink_bindings
from tink.jwt import _jwt_format
from tink.jwt import _jwt_hmac_key_manager
from tink.jwt import _jwt_mac


DATETIME_1970 = datetime.datetime.fromtimestamp(12345, datetime.timezone.utc)
DATETIME_2020 = datetime.datetime.fromtimestamp(1582230020,
                                                datetime.timezone.utc)


def setUpModule():
  _jwt_hmac_key_manager.register()


def _fixed_key_data() -> tink_pb2.KeyData:
  # test example in https://tools.ietf.org/html/rfc7515#appendix-A.1.1
  key_encoded = (b'AyM1SysPpbyDfgZld3umj1qzKObwVMkoqQ-EstJQLr_'
                 b'T-1qS0gZH75aKtMN3Yj0iPS4hcgUuTwjAzZr1Z9CAow')
  padded_key_encoded = key_encoded + b'=' * (-len(key_encoded) % 4)
  key_value = base64.urlsafe_b64decode(padded_key_encoded)
  jwt_hmac_key = jwt_hmac_pb2.JwtHmacKey(
      version=0, algorithm=jwt_hmac_pb2.HS256, key_value=key_value)
  return tink_pb2.KeyData(
      type_url='type.googleapis.com/google.crypto.tink.JwtHmacKey',
      key_material_type=tink_pb2.KeyData.SYMMETRIC,
      value=jwt_hmac_key.SerializeToString())


def _cc_mac() -> Any:
  key_data = _fixed_key_data()
  cc_key_manager = tink_bindings.MacKeyManager.from_cc_registry(
      'type.googleapis.com/google.crypto.tink.JwtHmacKey')
  return cc_key_manager.primitive(key_data.SerializeToString())


def create_fixed_jwt_hmac() -> _jwt_mac.JwtMacInternal:
  key_data = _fixed_key_data()
  key_manager = _jwt_hmac_key_manager.MacCcToPyJwtMacKeyManager()
  return key_manager.primitive(key_data)


def gen_token(json_header: str, json_payload: str) -> str:
  cc_mac = _cc_mac()
  unsigned_token = (
      _jwt_format.encode_header(json_header) + b'.' +
      _jwt_format.encode_payload(json_payload))
  return _jwt_format.create_signed_compact(unsigned_token,
                                           cc_mac.compute_mac(unsigned_token))


class JwtHmacKeyManagerTest(parameterized.TestCase):

  def test_basic(self):
    key_manager = _jwt_hmac_key_manager.MacCcToPyJwtMacKeyManager()
    self.assertEqual(key_manager.primitive_class(), _jwt_mac.JwtMacInternal)
    self.assertEqual(key_manager.key_type(),
                     'type.googleapis.com/google.crypto.tink.JwtHmacKey')

  @parameterized.named_parameters([
      ('JWT_HS256', jwt.raw_jwt_hs256_template()),
      ('JWT_HS384', jwt.raw_jwt_hs384_template()),
      ('JWT_HS512', jwt.raw_jwt_hs512_template()),
  ])
  def test_new_keydata_primitive_success(self, template):
    key_manager = _jwt_hmac_key_manager.MacCcToPyJwtMacKeyManager()
    key_data = key_manager.new_key_data(template)
    jwt_hmac = key_manager.primitive(key_data)

    raw_jwt = jwt.new_raw_jwt(
        type_header='typeHeader', issuer='issuer', without_expiration=True)
    validator = jwt.new_validator(
        expected_type_header='typeHeader',
        expected_issuer='issuer',
        allow_missing_expiration=True,
        fixed_now=DATETIME_1970)

    token_with_kid = jwt_hmac.compute_mac_and_encode_with_kid(
        raw_jwt, kid='kid-123')
    token_without_kid = jwt_hmac.compute_mac_and_encode_with_kid(
        raw_jwt, kid=None)

    # Verification of a token with a kid only fails if the wrong kid is passed.
    verified_jwt = jwt_hmac.verify_mac_and_decode_with_kid(
        token_with_kid, validator, kid='kid-123')
    self.assertEqual(verified_jwt.type_header(), 'typeHeader')
    self.assertEqual(verified_jwt.issuer(), 'issuer')
    jwt_hmac.verify_mac_and_decode_with_kid(token_with_kid, validator, kid=None)
    with self.assertRaises(tink.TinkError):
      jwt_hmac.verify_mac_and_decode_with_kid(
          token_with_kid, validator, kid='other-kid')

    # A token without kid is only valid if no kid is passed.
    jwt_hmac.verify_mac_and_decode_with_kid(
        token_without_kid, validator, kid=None)
    with self.assertRaises(tink.TinkError):
      jwt_hmac.verify_mac_and_decode_with_kid(
          token_without_kid, validator, kid='kid-123')

  def test_fixed_signed_compact(self):
    signed_compact = (
        'eyJ0eXAiOiJKV1QiLA0KICJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJqb2UiLA0KICJleH'
        'AiOjEzMDA4MTkzODAsDQogImh0dHA6Ly9leGFtcGxlLmNvbS9pc19yb290Ijp0cnVlfQ.'
        'dBjftJeZ4CVP-mB92K27uhbUJU1p1r_wW1gFWFOEjXk')
    jwt_hmac = create_fixed_jwt_hmac()
    verified_jwt = jwt_hmac.verify_mac_and_decode_with_kid(
        signed_compact,
        jwt.new_validator(
            expected_type_header='JWT',
            expected_issuer='joe',
            fixed_now=DATETIME_1970),
        kid=None)
    self.assertEqual(verified_jwt.issuer(), 'joe')
    self.assertEqual(verified_jwt.expiration().year, 2011)
    self.assertCountEqual(verified_jwt.custom_claim_names(),
                          ['http://example.com/is_root'])
    self.assertTrue(verified_jwt.custom_claim('http://example.com/is_root'))
    self.assertTrue(verified_jwt.type_header(), 'JWT')

    # fails because it is expired
    with self.assertRaises(tink.TinkError):
      jwt_hmac.verify_mac_and_decode_with_kid(
          signed_compact, jwt.new_validator(fixed_now=DATETIME_2020), kid=None)

    # fails with wrong issuer
    with self.assertRaises(tink.TinkError):
      jwt_hmac.verify_mac_and_decode_with_kid(
          signed_compact,
          jwt.new_validator(expected_issuer='jane', fixed_now=DATETIME_1970),
          kid=None)

  def test_weird_tokens_with_valid_macs(self):
    jwt_hmac = create_fixed_jwt_hmac()
    validator = jwt.new_validator(
        expected_issuer='joe', allow_missing_expiration=True)
    cc_mac = _cc_mac()

    # Normal token.
    valid_token = gen_token('{"alg":"HS256"}', '{"iss":"joe"}')
    verified = jwt_hmac.verify_mac_and_decode_with_kid(
        valid_token, validator, kid=None)
    self.assertEqual(verified.issuer(), 'joe')

    # Token with unknown header is valid.
    token_with_unknown_header = gen_token(
        '{"unknown_header":"123","alg":"HS256"}', '{"iss":"joe"}')
    verified2 = jwt_hmac.verify_mac_and_decode_with_kid(
        token_with_unknown_header, validator, kid=None)
    self.assertEqual(verified2.issuer(), 'joe')

    # Token with unknown kid is valid, since primitives with output prefix type
    # RAW ignore kid headers.
    token_with_unknown_kid = gen_token('{"kid":"unknown","alg":"HS256"}',
                                       '{"iss":"joe"}')
    verified2 = jwt_hmac.verify_mac_and_decode_with_kid(
        token_with_unknown_kid, validator, kid=None)
    self.assertEqual(verified2.issuer(), 'joe')

    # Token with invalid alg header
    alg_invalid = gen_token('{"alg":"HS384"}', '{"iss":"joe"}')
    with self.assertRaises(tink.TinkError):
      jwt_hmac.verify_mac_and_decode_with_kid(alg_invalid, validator, kid=None)

    # Token with empty header
    empty_header = gen_token('{}', '{"iss":"joe"}')
    with self.assertRaises(tink.TinkError):
      jwt_hmac.verify_mac_and_decode_with_kid(empty_header, validator, kid=None)

    # Token header is not valid JSON
    header_invalid = gen_token('{"alg":"HS256"', '{"iss":"joe"}')
    with self.assertRaises(tink.TinkError):
      jwt_hmac.verify_mac_and_decode_with_kid(
          header_invalid, validator, kid=None)

    # Token payload is not valid JSON
    payload_invalid = gen_token('{"alg":"HS256"}', '{"iss":"joe"')
    with self.assertRaises(tink.TinkError):
      jwt_hmac.verify_mac_and_decode_with_kid(
          payload_invalid, validator, kid=None)

    # Token with whitespace in header JSON string is valid.
    whitespace_in_header = gen_token(' {"alg":   \n  "HS256"} \n ',
                                     '{"iss":"joe" }')
    verified_jwt = jwt_hmac.verify_mac_and_decode_with_kid(
        whitespace_in_header, validator, kid=None)
    self.assertEqual(verified_jwt.issuer(), 'joe')

    # Token with whitespace in payload JSON string is valid.
    whitespace_in_payload = gen_token('{"alg":"HS256"}',
                                      ' {"iss": \n"joe" } \n')
    verified_jwt = jwt_hmac.verify_mac_and_decode_with_kid(
        whitespace_in_payload, validator, kid=None)
    self.assertEqual(verified_jwt.issuer(), 'joe')

    # Token with whitespace in base64-encoded header is invalid.
    with_whitespace_in_encoding = (
        _jwt_format.encode_header('{"alg":"HS256"}') + b' .' +
        _jwt_format.encode_payload('{"iss":"joe"}'))
    token_with_whitespace_in_encoding = _jwt_format.create_signed_compact(
        with_whitespace_in_encoding,
        cc_mac.compute_mac(with_whitespace_in_encoding))
    with self.assertRaises(tink.TinkError):
      jwt_hmac.verify_mac_and_decode_with_kid(
          token_with_whitespace_in_encoding, validator, kid=None)

    # Token with invalid character is invalid.
    with_invalid_char = (
        _jwt_format.encode_header('{"alg":"HS256"}') + b'.?' +
        _jwt_format.encode_payload('{"iss":"joe"}'))
    token_with_invalid_char = _jwt_format.create_signed_compact(
        with_invalid_char, cc_mac.compute_mac(with_invalid_char))
    with self.assertRaises(tink.TinkError):
      jwt_hmac.verify_mac_and_decode_with_kid(
          token_with_invalid_char, validator, kid=None)

    # Token with additional '.' is invalid.
    with_dot = (
        _jwt_format.encode_header('{"alg":"HS256"}') + b'.' +
        _jwt_format.encode_payload('{"iss":"joe"}') + b'.')
    token_with_dot = _jwt_format.create_signed_compact(
        with_dot, cc_mac.compute_mac(with_dot))
    with self.assertRaises(tink.TinkError):
      jwt_hmac.verify_mac_and_decode_with_kid(
          token_with_dot, validator, kid=None)

    # num_recursions has been chosen such that parsing of this token fails
    # in all languages. We want to make sure that the algorithm does not
    # hang or crash in this case, but only returns a parsing error.
    num_recursions = 10000
    rec_payload = ('{"a":' * num_recursions) + '""' + ('}' * num_recursions)
    rec_token = gen_token('{"alg":"HS256"}', rec_payload)
    with self.assertRaises(tink.TinkError):
      jwt_hmac.verify_mac_and_decode_with_kid(
          rec_token,
          validator=jwt.new_validator(allow_missing_expiration=True),
          kid=None)

    # test wrong types
    with self.assertRaises(tink.TinkError):
      jwt_hmac.verify_mac_and_decode_with_kid(
          cast(str, None), validator, kid=None)
    with self.assertRaises(tink.TinkError):
      jwt_hmac.verify_mac_and_decode_with_kid(
          cast(str, 123), validator, kid=None)
    with self.assertRaises(tink.TinkError):
      valid_token_bytes = valid_token.encode('utf8')
      jwt_hmac.verify_mac_and_decode_with_kid(
          cast(str, valid_token_bytes), validator, kid=None)

  @parameterized.named_parameters([
      ('modified_signature',
       ('eyJ0eXAiOiJKV1QiLA0KICJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJqb2UiLA0KICJleH'
        'AiOjEzMDA4MTkzODAsDQogImh0dHA6Ly9leGFtcGxlLmNvbS9pc19yb290Ijp0cnVlfQ.'
        'dBjftJeZ4CVP-mB92K27uhbUJU1p1r_wW1gFWFOEjXi')),
      ('modified_payload',
       ('eyJ0eXAiOiJKV1QiLA0KICJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJqb2UiLCJleHAiOj'
        'EzMDA4MTkzODEsImh0dHA6Ly9leGFtcGxlLmNvbS9pc19yb290Ijp0cnVlfQ.'
        'dBjftJeZ4CVP-mB92K27uhbUJU1p1r_wW1gFWFOEjXk')),
      ('modified_header',
       ('eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJqb2UiLA0KICJleH'
        'AiOjEzMDA4MTkzODAsDQogImh0dHA6Ly9leGFtcGxlLmNvbS9pc19yb290Ijp0cnVlfQ.'
        'dBjftJeZ4CVP-mB92K27uhbUJU1p1r_wW1gFWFOEjXk')),
      ('extra .', 'eyJhbGciOiJIUzI1NiJ9.e30.abc.'),
      ('invalid_header_encoding', 'eyJhbGciOiJIUzI1NiJ9?.e30.abc'),
      ('invalid_payload_encoding', 'eyJhbGciOiJIUzI1NiJ9.e30?.abc'),
      ('invalid_mac_encoding', 'eyJhbGciOiJIUzI1NiJ9.e30.abc?'),
      ('no_mac', 'eyJhbGciOiJIUzI1NiJ9.e30'),
  ])
  def test_invalid_signed_compact(self, invalid_signed_compact):
    jwt_hmac = create_fixed_jwt_hmac()
    validator = jwt.new_validator(
        expected_issuer='joe',
        allow_missing_expiration=True,
        fixed_now=DATETIME_1970)

    with self.assertRaises(tink.TinkError):
      jwt_hmac.verify_mac_and_decode_with_kid(
          invalid_signed_compact, validator, kid=None)


if __name__ == '__main__':
  absltest.main()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.testing.helper."""

import os

from absl.testing import absltest
from tink import core
from tink.testing import helper


class HelperTest(absltest.TestCase):

  def test_tink_py_testdata_path(self):
    path = os.path.join(helper.tink_py_testdata_path(), 'gcp/credential.json')
    with open(path, mode='rt') as f:
      credential_json = f.read()
    self.assertNotEmpty(credential_json)

  def test_fake_mac_success(self):
    mac = helper.FakeMac('Name')
    mac_value = mac.compute_mac(b'data')
    mac.verify_mac(mac_value, b'data')

  def test_fake_mac_fail_wrong_data(self):
    mac = helper.FakeMac('Name')
    mac_value = mac.compute_mac(b'data')
    with self.assertRaises(core.TinkError):
      mac.verify_mac(mac_value, b'wrong data')

  def test_fake_mac_fail_wrong_primitive(self):
    mac = helper.FakeMac('Name')
    mac_value = mac.compute_mac(b'data')
    wrong_mac = helper.FakeMac('Wrong Name')
    with self.assertRaises(core.TinkError):
      wrong_mac.verify_mac(mac_value, b'data')

  def test_fake_aead_success(self):
    aead = helper.FakeAead('Name')
    ciphertext = aead.encrypt(b'plaintext', b'associated_data')
    self.assertEqual(
        aead.decrypt(ciphertext, b'associated_data'),
        b'plaintext')

  def test_fake_aead_fail_wrong_cipertext(self):
    aead = helper.FakeAead('Name')
    with self.assertRaises(core.TinkError):
      aead.decrypt(b'wrong ciphertext', b'associated_data')

  def test_fake_aead_fail_wrong_associated_data(self):
    aead = helper.FakeAead('Name')
    ciphertext = aead.encrypt(b'plaintext', b'associated_data')
    with self.assertRaises(core.TinkError):
      aead.decrypt(ciphertext, b'wrong_associated_data')

  def test_fake_aead_fail_wrong_primitive(self):
    aead = helper.FakeAead('Name')
    ciphertext = aead.encrypt(b'plaintext', b'associated_data')
    wrong_aead = helper.FakeAead('Wrong Name')
    with self.assertRaises(core.TinkError):
      wrong_aead.decrypt(ciphertext, b'associated_data')

  def test_fake_deterministic_aead_success(self):
    daead = helper.FakeDeterministicAead('Name')
    ciphertext = daead.encrypt_deterministically(b'plaintext',
                                                 b'associated_data')
    self.assertEqual(
        daead.decrypt_deterministically(ciphertext, b'associated_data'),
        b'plaintext')

  def test_fake_deterministic_aead_fail_wrong_cipertext(self):
    daead = helper.FakeDeterministicAead('Name')
    with self.assertRaises(core.TinkError):
      daead.decrypt_deterministically(b'wrong ciphertext', b'associated_data')

  def test_fake_deterministic_aead_fail_wrong_associated_data(self):
    daead = helper.FakeDeterministicAead('Name')
    ciphertext = daead.encrypt_deterministically(b'plaintext',
                                                 b'associated_data')
    with self.assertRaises(core.TinkError):
      daead.decrypt_deterministically(ciphertext, b'wrong_associated_data')

  def test_fake_deterministic_aead_fail_wrong_primitive(self):
    daead = helper.FakeDeterministicAead('Name')
    ciphertext = daead.encrypt_deterministically(b'plaintext',
                                                 b'associated_data')
    wrong_daead = helper.FakeDeterministicAead('Wrong Name')
    with self.assertRaises(core.TinkError):
      wrong_daead.decrypt_deterministically(ciphertext, b'associated_data')

  def test_fake_hybrid_success(self):
    enc = helper.FakeHybridEncrypt('Name')
    dec = helper.FakeHybridDecrypt('Name')
    ciphertext = enc.encrypt(b'plaintext', b'context_info')
    self.assertEqual(
        dec.decrypt(ciphertext, b'context_info'),
        b'plaintext')

  def test_fake_hybrid_fail_wrong_context(self):
    enc = helper.FakeHybridEncrypt('Name')
    dec = helper.FakeHybridDecrypt('Name')
    ciphertext = enc.encrypt(b'plaintext', b'context_info')
    with self.assertRaises(core.TinkError):
      dec.decrypt(ciphertext, b'other_context_info')

  def test_fake_hybrid_fail_wrong_dec(self):
    enc = helper.FakeHybridEncrypt('Name')
    dec = helper.FakeHybridDecrypt('Wrong Name')
    ciphertext = enc.encrypt(b'plaintext', b'context_info')
    with self.assertRaises(core.TinkError):
      dec.decrypt(ciphertext, b'context_info')

  def test_fake_hybrid_fail_wrong_ciphertext(self):
    dec = helper.FakeHybridDecrypt('Name')
    with self.assertRaises(core.TinkError):
      dec.decrypt(b'wrong ciphertext', b'context_info')

  def test_fake_prf_set_success(self):
    prf_set = helper.FakePrfSet('Name')
    output_primary = prf_set.primary().compute(b'input', output_length=31)
    self.assertLen(output_primary, 31)
    prfs = prf_set.all()
    self.assertLen(prfs, 1)
    output = prfs[0].compute(b'input', output_length=31)
    self.assertLen(output, 31)


if __name__ == '__main__':
  absltest.main()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Interface for JwtPublicKeyVerify."""

import abc

from typing import Optional

from tink.jwt import _jwt_validator
from tink.jwt import _verified_jwt


class JwtPublicKeyVerify(metaclass=abc.ABCMeta):
  """Interface for verifying a signed JWT.

  Sees RFC 7519 and RFC 7515. Security guarantees: similar to PublicKeyVerify.
  """

  @abc.abstractmethod
  def verify_and_decode(
      self, compact: str,
      validator: _jwt_validator.JwtValidator) -> _verified_jwt.VerifiedJwt:
    """Verifies, validates and decodes a signed compact JWT token.

    The JWT is validated against the rules in validator. That is, every claim in
    validator must also be present in the JWT. For example, if validator
    contains an issuer (iss) claim, the JWT must contain an identical claim.
    The JWT can contain claims that are NOT in the validator, which are then
    ignored. However, if the JWT contains a list of audiences, the validator
    must also contain an audience in the list.

    If the JWT contains timestamp claims such as expiration (exp), issued_at
    (iat) or not_before (nbf), they will also be validated. Validator allows to
    set a clock skew, to deal with small clock differences among different
    machines.

    Args:
      compact: A signed token encoded in the JWS compact serialization format.
      validator: A JwtValidator that validates the token.

    Returns:
      A VerifiedJwt.
    Raises:
      tink.TinkError if the operation fails.
    """
    raise NotImplementedError()


class JwtPublicKeyVerifyInternal(metaclass=abc.ABCMeta):
  """Internal interface for creating a signed JWT.

  "kid" is an optional value that is set by the wrapper for keys with output
  prefix TINK. It is set to None for output prefix RAW.
  """

  @abc.abstractmethod
  def verify_and_decode_with_kid(
      self, compact: str, validator: _jwt_validator.JwtValidator,
      kid: Optional[str]) -> _verified_jwt.VerifiedJwt:
    raise NotImplementedError()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Pre-generated KeyTemplates for StreamingAead.


Currently, these templates cannot be used to generate keysets, but they can be
used to generate individual keys.
In the future, it will be possible to use these templates to generate a new
tink_pb2.Keyset with tink_pb2.KeysetHandle. To generate a new keyset that
contains a single aes_ctr_hmac_streaming_pb2.AesCtrHmacStreamingKey, one can do:
handle = keyset_handle.KeysetHandle(
  streaming_aead_key_templates.AES256_CTR_HMAC_SHA256_4KB).
"""

import warnings

from tink.proto import aes_ctr_hmac_streaming_pb2
from tink.proto import aes_gcm_hkdf_streaming_pb2
from tink.proto import common_pb2
from tink.proto import tink_pb2

_AES_GCM_HKDF_STREAMING_KEY_TYPE_URL = (
    'type.googleapis.com/google.crypto.tink.AesGcmHkdfStreamingKey')
_AES_CTR_HMAC_STREAMING_KEY_TYPE_URL = (
    'type.googleapis.com/google.crypto.tink.AesCtrHmacStreamingKey')
SEGMENT_SIZE_1MB = 1024 * 1024
SEGMENT_SIZE_4KB = 4 * 1024


def _create_aes_gcm_hkdf_streaming_key_template(
    aes_key_size: int, hash_type: common_pb2.HashType, derived_key_size: int,
    ciphertext_segment_size: int) -> tink_pb2.KeyTemplate:
  """Creates an AES GCM HKDF Streaming KeyTemplate, and fills in its values."""
  key_format = aes_gcm_hkdf_streaming_pb2.AesGcmHkdfStreamingKeyFormat()
  key_format.key_size = aes_key_size
  key_format.params.hkdf_hash_type = hash_type
  key_format.params.derived_key_size = derived_key_size
  key_format.params.ciphertext_segment_size = ciphertext_segment_size

  key_template = tink_pb2.KeyTemplate(
      value=key_format.SerializeToString(),
      type_url=_AES_GCM_HKDF_STREAMING_KEY_TYPE_URL,
      output_prefix_type=tink_pb2.RAW,
  )
  return key_template


def _create_aes_ctr_hmac_streaming_key_template(
    aes_key_size: int, hkdf_hash_type: common_pb2.HashType,
    derived_key_size: int, mac_hash_type: common_pb2.HashType, tag_size: int,
    ciphertext_segment_size: int) -> tink_pb2.KeyTemplate:
  """Creates an AES CTR HMAC Streaming KeyTemplate, and fills in its values."""
  key_format = aes_ctr_hmac_streaming_pb2.AesCtrHmacStreamingKeyFormat()
  key_format.key_size = aes_key_size

  key_format.params.ciphertext_segment_size = ciphertext_segment_size
  key_format.params.derived_key_size = derived_key_size
  key_format.params.hkdf_hash_type = hkdf_hash_type

  key_format.params.hmac_params.hash = mac_hash_type
  key_format.params.hmac_params.tag_size = tag_size

  key_template = tink_pb2.KeyTemplate(
      value=key_format.SerializeToString(),
      type_url=_AES_CTR_HMAC_STREAMING_KEY_TYPE_URL,
      output_prefix_type=tink_pb2.RAW,
  )
  return key_template


AES128_GCM_HKDF_4KB = _create_aes_gcm_hkdf_streaming_key_template(
    aes_key_size=16,
    hash_type=common_pb2.HashType.SHA256,
    derived_key_size=16,
    ciphertext_segment_size=SEGMENT_SIZE_4KB)

AES128_GCM_HKDF_1MB = _create_aes_gcm_hkdf_streaming_key_template(
    aes_key_size=16,
    hash_type=common_pb2.HashType.SHA256,
    derived_key_size=16,
    ciphertext_segment_size=SEGMENT_SIZE_1MB)

AES256_GCM_HKDF_4KB = _create_aes_gcm_hkdf_streaming_key_template(
    aes_key_size=32,
    hash_type=common_pb2.HashType.SHA256,
    derived_key_size=32,
    ciphertext_segment_size=SEGMENT_SIZE_4KB)

AES256_GCM_HKDF_1MB = _create_aes_gcm_hkdf_streaming_key_template(
    aes_key_size=32,
    hash_type=common_pb2.HashType.SHA256,
    derived_key_size=32,
    ciphertext_segment_size=SEGMENT_SIZE_1MB)

AES128_CTR_HMAC_SHA256_4KB = _create_aes_ctr_hmac_streaming_key_template(
    aes_key_size=16,
    hkdf_hash_type=common_pb2.HashType.SHA256,
    derived_key_size=16,
    mac_hash_type=common_pb2.HashType.SHA256,
    tag_size=32,
    ciphertext_segment_size=SEGMENT_SIZE_4KB)

AES128_CTR_HMAC_SHA256_1MB = _create_aes_ctr_hmac_streaming_key_template(
    aes_key_size=16,
    hkdf_hash_type=common_pb2.HashType.SHA256,
    derived_key_size=16,
    mac_hash_type=common_pb2.HashType.SHA256,
    tag_size=32,
    ciphertext_segment_size=SEGMENT_SIZE_1MB)

AES256_CTR_HMAC_SHA256_4KB = _create_aes_ctr_hmac_streaming_key_template(
    aes_key_size=32,
    hkdf_hash_type=common_pb2.HashType.SHA256,
    derived_key_size=32,
    mac_hash_type=common_pb2.HashType.SHA256,
    tag_size=32,
    ciphertext_segment_size=SEGMENT_SIZE_4KB)

AES256_CTR_HMAC_SHA256_1MB = _create_aes_ctr_hmac_streaming_key_template(
    aes_key_size=32,
    hkdf_hash_type=common_pb2.HashType.SHA256,
    derived_key_size=32,
    mac_hash_type=common_pb2.HashType.SHA256,
    tag_size=32,
    ciphertext_segment_size=SEGMENT_SIZE_1MB)


# Deprecated. Use the predefined constant templates above instead.
def create_aes_gcm_hkdf_streaming_key_template(
    aes_key_size: int, hash_type: common_pb2.HashType, derived_key_size: int,
    ciphertext_segment_size: int) -> tink_pb2.KeyTemplate:
  warnings.warn(
      'The create_aes_gcm_hkdf_streaming_key_template function is deprecated.',
      DeprecationWarning, 2)
  return _create_aes_gcm_hkdf_streaming_key_template(aes_key_size, hash_type,
                                                     derived_key_size,
                                                     ciphertext_segment_size)


# Deprecated. Use the predefined constant templates above instead.
def create_aes_ctr_hmac_streaming_key_template(
    aes_key_size: int, hkdf_hash_type: common_pb2.HashType,
    derived_key_size: int, mac_hash_type: common_pb2.HashType, tag_size: int,
    ciphertext_segment_size: int) -> tink_pb2.KeyTemplate:
  warnings.warn(
      'The create_aes_ctr_hmac_streaming_key_template function is deprecated.',
      DeprecationWarning, 2)
  return _create_aes_ctr_hmac_streaming_key_template(aes_key_size,
                                                     hkdf_hash_type,
                                                     derived_key_size,
                                                     mac_hash_type, tag_size,
                                                     ciphertext_segment_size)
